package com.caimei.modules.homepage.controller;

import com.caimei.modules.club.entity.Page;
import com.caimei.modules.homepage.enity.*;
import com.caimei.modules.homepage.service.PageService;
import com.caimei.utils.ResponseJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.junit.Assert;
import org.springframework.web.bind.annotation.*;

/**
 * @author zzj
 */
@Api(tags = "首页api")
@RestController
@RequiredArgsConstructor
@RequestMapping("/home/page")
public class CmHomePageController {

    private final PageService pageService;

    @ApiOperation("保存楼层")
    @PostMapping("/save/floor")
    public ResponseJson saveTheme(@RequestBody Floor floor) {
        Assert.assertNotNull("组织Id不能为空！", floor.getOrganizeId());
        return ResponseJson.success(pageService.saveFloor(floor));
    }

    @ApiOperation("删除楼层")
    @GetMapping("/del/floor/{id}")
    public ResponseJson delFloor(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.delFloor(id));
    }

    @ApiOperation("楼层排序")
    @GetMapping("/floor/{id}/{sort}")
    public ResponseJson floorSort(@PathVariable("id") Integer id, @PathVariable("sort") Integer sort) {
        return ResponseJson.success(pageService.sortChange(id, sort, 3));
    }

    @ApiOperation("上/下线楼层")
    @GetMapping("/check/floor/status/{id}")
    public ResponseJson checkFloorStatus(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.checkFloorStatus(id));
    }

    @ApiOperation("楼层列表")
    @GetMapping("/floor/list")
    public ResponseJson<Page<Floor>> themeList(Floor floor, Page page) {
        Assert.assertNotNull("组织Id不能为空！", floor.getOrganizeId());
        return ResponseJson.success(pageService.findFloorList(floor, page));
    }

    @ApiOperation("楼层详情")
    @GetMapping("/floor/form/{id}")
    public ResponseJson<Floor> floorForm(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.floorForm(id));
    }

    @ApiOperation("banner图排序")
    @GetMapping("/banner/{id}/{sort}")
    public ResponseJson bannerSort(@PathVariable("id") Integer id, @PathVariable("sort") Integer sort) {
        return ResponseJson.success(pageService.sortChange(id, sort, 1));
    }

    @ApiOperation("菜单排序")
    @GetMapping("/menu/{id}/{sort}")
    public ResponseJson menuSort(@PathVariable("id") Integer id, @PathVariable("sort") Integer sort) {
        return ResponseJson.success(pageService.sortChange(id, sort, 2));
    }

    @ApiOperation("专题活动列表")
    @GetMapping("/theme/list")
    public ResponseJson themeList(Theme theme, Page page) {
        Assert.assertNotNull("组织Id不能为空！", theme.getOrganizeId());
        return ResponseJson.success(pageService.findThemeList(theme, page));
    }

    @ApiOperation("专题活动停用/启用")
    @GetMapping("/check/theme/status/{id}")
    public ResponseJson checkThemeStatus(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.checkThemeStatus(id));
    }

    @ApiOperation("保存专题活动")
    @PostMapping("/save/theme")
    public ResponseJson saveTheme(@RequestBody Theme theme) {
        return ResponseJson.success(pageService.saveTheme(theme));
    }

    @ApiOperation("删除专题活动")
    @GetMapping("/del/theme/{id}")
    public ResponseJson delTheme(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.delTheme(id));
    }

    @ApiOperation("专题活动详情")
    @GetMapping("/theme/form/{id}")
    public ResponseJson<Theme> themeForm(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.themeForm(id));
    }

    @ApiOperation("banner图详情")
    @GetMapping("/banner/form/{id}")
    public ResponseJson<Banner> bannerForm(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.findBannerById(id));
    }

    @ApiOperation("banner图列表")
    @GetMapping("/banner/list")
    public ResponseJson bannerList(Banner banner, Page page) {
        Assert.assertNotNull("组织Id不能为空！", banner.getOrganizeId());
        return ResponseJson.success(pageService.findBannerList(banner, page));
    }

    @ApiOperation("保存banner图")
    @PostMapping("/save/banner")
    public ResponseJson addBanner(@RequestBody Banner banner) {
        Assert.assertNotNull("组织Id不能为空！", banner.getOrganizeId());
        return ResponseJson.success(pageService.saveBanner(banner));
    }

    @ApiOperation("banner图启用/停用")
    @GetMapping("/check/banner/status/{id}")
    public ResponseJson checkBannerStatus(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.checkStatus(id));
    }

    @ApiOperation("删除banner图")
    @GetMapping("/del/banner/{id}")
    public ResponseJson delBanner(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.deleteBanner(id));
    }

    @ApiOperation("菜单详情")
    @GetMapping("/menu/form/{id}")
    public ResponseJson<MainMenu> menuForm(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.findMenuById(id));
    }

    @ApiOperation("保存主菜单")
    @PostMapping("/save/menu")
    public ResponseJson saveMainMenu(@RequestBody MainMenu menu) {
        Assert.assertNotNull("组织Id不能为空！", menu.getOrganizeId());
        return ResponseJson.success(pageService.saveMenu(menu));
    }

    @ApiOperation("主菜单列表")
    @GetMapping("/menu/list")
    public ResponseJson menuList(MainMenu menu, Page page) {
        Assert.assertNotNull("组织Id不能为空！", menu.getOrganizeId());
        return ResponseJson.success(pageService.findMenuList(menu, page));
    }

    @ApiOperation("主菜单启用/停用")
    @GetMapping("/check/menu/status/{id}")
    public ResponseJson checkMenuStatus(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.checkMenuStatus(id));
    }

    @ApiOperation("删除主菜单")
    @GetMapping("/del/menu/{id}")
    public ResponseJson delMenu(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.deleteMenu(id));
    }

    @ApiOperation("保存活动")
    @PostMapping("/save/active")
    public ResponseJson saveActive(@RequestBody Active active) {
        Assert.assertNotNull("组织Id不能为空！", active.getOrganizeId());
        return ResponseJson.success(pageService.saveActive(active));
    }

    @ApiOperation("活动列表")
    @GetMapping("/active/list")
    public ResponseJson<Page<Active>> activeList(Active active, Page page) {
        Assert.assertNotNull("组织Id不能为空！", active.getOrganizeId());
        return ResponseJson.success(pageService.findActiveList(active, page));
    }

    @ApiOperation("活动启用/停用")
    @GetMapping("/check/active/status/{id}")
    public ResponseJson checkActiveStatus(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.checkActiveStatus(id));
    }

    @ApiOperation("删除活动")
    @GetMapping("/del/active/{id}")
    public ResponseJson delActive(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.delActive(id));
    }

    @ApiOperation("活动详情")
    @GetMapping("/active/form/{id}")
    public ResponseJson<Active> activeForm(@PathVariable("id") Integer id) {
        return ResponseJson.success(pageService.activeForm(id));
    }
}
