package com.caimei.modules.homepage.service.impl;

import com.caimei.modules.club.entity.Page;
import com.caimei.modules.homepage.dao.*;
import com.caimei.modules.homepage.enity.*;
import com.caimei.modules.homepage.service.PageService;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author zzj
 */
@Service
@Slf4j
public class PageServiceImpl implements PageService {

    @Resource
    private BannerDao bannerDao;
    @Resource
    private MainMenuDao menuDao;
    @Resource
    private ActiveDao activeDao;
    @Resource
    private ThemeDao themeDao;
    @Resource
    private FloorDao floorDao;

    @Override
    public Page<Banner> findBannerList(Banner banner, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<Banner> bannerList = bannerDao.findBannerList(banner);
        return new Page<>(bannerList);
    }

    @Override
    public ResponseJson saveBanner(Banner banner) {
        if (null != banner.getId()) {
            bannerDao.updateBanner(banner);
            return ResponseJson.success("修改成功！");
        }
        bannerDao.insertBanner(banner);
        return ResponseJson.success("添加成功!");
    }

    @Override
    public ResponseJson checkStatus(Integer id) {
        bannerDao.checkBannerStatus(id);
        return ResponseJson.success(ResponseJson.success("成功！"));
    }

    @Override
    public ResponseJson deleteBanner(Integer id) {
        bannerDao.delBanner(id);
        return ResponseJson.success("删除成功!");
    }

    @Override
    public ResponseJson saveMenu(MainMenu menu) {
        if (null != menu.getId()) {
            menuDao.updateById(menu);
            return ResponseJson.success("修改成功!");
        }
        menu.setAddTime(new Date());
        menuDao.insert(menu);
        return ResponseJson.success("添加成功！");
    }

    @Override
    public Page<MainMenu> findMenuList(MainMenu menu, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<MainMenu> list = menuDao.findList(menu);
        return new Page<>(list);
    }

    @Override
    public ResponseJson checkMenuStatus(Integer id) {
        menuDao.checkMenuStatus(id);
        return ResponseJson.success("成功！");
    }

    @Override
    public ResponseJson deleteMenu(Integer id) {
        menuDao.deleteById(id);
        return ResponseJson.success("删除成功！");
    }

    @Override
    public ResponseJson saveActive(Active active) {
        if (null != active.getId()) {
            activeDao.delProducts(active.getId());
            activeDao.delImages(active.getId());
            activeDao.updateActive(active);
        } else {
            activeDao.insertActive(active);
        }
        /**
         * type 为1时，仅需保存productId
         */
        if (1 != active.getActiveType()) {
            active.getImages().forEach(i -> {
                i.setActiveId(active.getId());
                activeDao.insertImages(i);
                i.getProducts().forEach(p -> activeDao.insertImageProduct(active.getId(), i.getId(), p.getProductId(), p.getSort()));
            });
            return ResponseJson.success("保存成功！");
        }
        active.getProducts().forEach(p -> activeDao.insertProducts(active.getId(), p.getProductId(), p.getSort()));
        return ResponseJson.success("保存成功！");
    }

    @Override
    public Page<Active> findActiveList(Active active, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<Active> activeList = activeDao.findActiveList(active);
        return new Page<>(activeList);
    }

    @Override
    public ResponseJson checkActiveStatus(Integer id) {
        activeDao.checkActiveStatus(id);
        return ResponseJson.success("成功!");
    }

    @Override
    public ResponseJson delActive(Integer id) {
        activeDao.delActive(id);
        activeDao.delActiveImage(id);
        activeDao.delActiveProduct(id);
        return ResponseJson.success("删除成功");
    }

    @Override
    public Banner findBannerById(Integer id) {
        Banner banner = bannerDao.findBannerById(id);
        if (1 == banner.getJumpType()) {
            banner.setProducts(bannerDao.findProducts(banner.getJumpProductId()));
        }
        return banner;
    }

    @Override
    public MainMenu findMenuById(Integer id) {
        MainMenu mainMenu = menuDao.selectById(id);
        if (1 == mainMenu.getJumpType()) {
            mainMenu.setProducts(bannerDao.findProducts(mainMenu.getJumpProductId()));
        }
        return mainMenu;
    }

    @Override
    public Page<Theme> findThemeList(Theme theme, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<Theme> list = themeDao.findList(theme);
        return new Page<>(list);
    }

    @Override
    public ResponseJson checkThemeStatus(Integer id) {
        Theme theme = themeDao.selectById(id);
        // 0停用 1启用
        if (0 == theme.getEnabledStatus()) {
            themeDao.closeAll();
        }
        themeDao.checkStatus(id);
        return ResponseJson.success("成功!");
    }

    @Override
    public ResponseJson saveTheme(Theme theme) {
        /**
         * 只允许一个启用的专题活动
         *  状态0停用 1启用
         *  本次保存为启用，则停用其他
         */
        if (1 == theme.getEnabledStatus()) {
            themeDao.closeAll();
        }
        if (null != theme.getId()) {
            themeDao.updateById(theme);
            return ResponseJson.success("修改成功!");
        }
        theme.setAddTime(new Date());
        themeDao.insert(theme);
        return ResponseJson.success("添加成功！");
    }

    @Override
    public ResponseJson delTheme(Integer id) {
        themeDao.deleteById(id);
        return ResponseJson.success("删除成功");
    }

    @Override
    public Theme themeForm(Integer id) {
        Theme theme = themeDao.selectById(id);
        if (1 == theme.getJumpType()) {
            theme.setProducts(bannerDao.findProducts(theme.getJumpProductId()));
        }
        return theme;
    }

    @Override
    public Active activeForm(Integer id) {
        Active active = activeDao.findActiveById(id);
        if (1 == active.getActiveType()) {
            active.setProducts(activeDao.findProducts(id));
        } else {
            List<ActiveImages> imageList = activeDao.findImageList(id);
            imageList.forEach(i -> i.setProducts(activeDao.findProductsByImage(i.getId())));
            active.setImages(imageList);
        }
        return active;
    }

    @Override
    public ResponseJson sortChange(Integer id, Integer sort, Integer status) {
        /**
         *  status 1  banner
         *  status 2  menu
         *  status 3  floor
         */
        bannerDao.updateSort(id, sort, status);
        return ResponseJson.success("修改排序成功！");
    }

    @Override
    public ResponseJson saveFloor(Floor floor) {
        if (null != floor.getId()) {
            floorDao.updateById(floor);
            floorDao.delProduct(floor.getId());
            floor.getProducts().forEach(p -> floorDao.insertFloorProduct(floor.getId(), p.getProductId(), p.getSort()));
            return ResponseJson.success("修改成功!");
        }
        floorDao.insertFloor(floor);
        floor.getProducts().forEach(p -> floorDao.insertFloorProduct(floor.getId(), p.getProductId(), p.getSort()));
        return ResponseJson.success("添加成功！");
    }

    @Override
    public ResponseJson delFloor(Integer id) {
        floorDao.deleteById(id);
        return ResponseJson.success("删除成功");
    }

    @Override
    public ResponseJson checkFloorStatus(Integer id) {
        floorDao.checkStatus(id);
        return ResponseJson.success("成功");
    }

    @Override
    public Page<Floor> findFloorList(Floor floor, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<Floor> list = floorDao.findList(floor);
        return new Page<>(list);
    }

    @Override
    public Floor floorForm(Integer id) {
        Floor floor = floorDao.selectById(id);
        floor.setProducts(floorDao.findProducts(id));
        return floor;
    }

}
