package com.caimei.modules.order.controller;

import com.caimei.modules.club.entity.Page;
import com.caimei.modules.order.entity.CmReturnedPurchase;
import com.caimei.modules.order.entity.NewOrder;
import com.caimei.modules.order.entity.NewShopOrder;
import com.caimei.modules.order.service.CmReturnedPurchaseService;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.JsonModel;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = "退货退款api")
@RequiredArgsConstructor
@RestController
@RequestMapping(value = "/order/cmReturnedPurchase")
public class CmReturnedPurchaseController {

    private final CmReturnedPurchaseService cmReturnedPurchaseService;

    private final NewOrderService newOrderService;

    private final OrderUtils orderUtil;


    @ApiOperation("用户退款列表")
    @GetMapping(value = "list")
    public JsonModel list(CmReturnedPurchase cmReturnedPurchase, Page page) {
        PageHelper.startPage(page.getIndex(), page.getPageSize());
        List<CmReturnedPurchase> returnedPurchaseList = cmReturnedPurchaseService.getReturnedPurchaseList(cmReturnedPurchase);
        Page<CmReturnedPurchase> purchasePage = new Page<>(returnedPurchaseList);
        return JsonModel.newInstance().success(purchasePage);
    }

    /**
     * 接口参数：
     * 用户撤销退款id
     * 组织organizeId
     * 主订单orderId
     * 子订单shopOrderId
     * @param cmReturnedPurchase
     * @return
     */
    @ApiOperation("用户撤销退款")
    @GetMapping(value = "revokeApply")
    public ResponseJson<String> revokeApply(CmReturnedPurchase cmReturnedPurchase) {
            cmReturnedPurchase = cmReturnedPurchaseService.get(cmReturnedPurchase.getId());
            if (!"2".equals(cmReturnedPurchase.getStatus())) {
               return ResponseJson.error("该退款单状态已发生改变，不可撤销");
            }
            cmReturnedPurchaseService.revokeApply(cmReturnedPurchase);

        return ResponseJson.success("已撤销退款");
    }


    @ApiOperation("用户退款单详情")
    @GetMapping(value = "/toReturnRecord/{id}")
    public ResponseJson<NewOrder> toReturnRecord(@PathVariable("id") Integer id) {
        CmReturnedPurchase cmReturnedPurchase = cmReturnedPurchaseService.get(id);
        NewOrder newOrder = newOrderService.get(cmReturnedPurchase.getOrderId().toString());
        //过滤子订单
        orderUtil.setOrderValue(newOrder);
        List<NewShopOrder>  newShopOrders= newOrder.getNewShopOrders();
//        .stream().filter(newShopOrder -> newShopOrder.getShopOrderId().equals(cmReturnedPurchase.getShopOrderId())).collect(Collectors.toList());
        newOrder.setNewShopOrders(newShopOrders);
        newOrder.setCmReturnedPurchase(cmReturnedPurchase);
        cmReturnedPurchaseService.toReturnRecord(newOrder, cmReturnedPurchase);
        return ResponseJson.success(newOrder);
    }

    @ApiOperation("退货退款详情")
    @GetMapping(value = "/returnDetail/{orderId}")
    public ResponseJson<List<NewOrder>> returnDetail(@PathVariable("orderId") Integer orderId) {
        return ResponseJson.success(cmReturnedPurchaseService.getReturnDetailList(orderId));
    }

    @ApiOperation("退付款记录")
    @GetMapping(value = "/returnAndPay/{shopOrderId}")
    public ResponseJson<Map<String, Object>> returnAndPay(@PathVariable("shopOrderId") Integer shopOrderId) {
        return ResponseJson.success(cmReturnedPurchaseService.getReturnAndPay(shopOrderId));
    }


}
