package com.caimei.modules.order.controller;


import com.caimei.modules.order.entity.ReceiptDetail;
import com.caimei.modules.order.entity.ReceiptPayType;
import com.caimei.modules.order.entity.ReceiptVo;
import com.caimei.modules.order.service.CmDiscernReceiptService;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageInfo;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author zzj
 */
@Api(tags = "订单收款API")
@ApiSupport(author = "zzj")
@RestController
@RequiredArgsConstructor
@RequestMapping(value = "/order/organize")
public class OrderReceiptController {

    private final CmDiscernReceiptService cmDiscernReceiptService;

    /**
     * 收款列表
     * 查询参数：收款Id 订单ID  订单编号 机构名称
     * 收款方式 收款进度 收款状态 关联数量 收款帐号 下单时间
     *
     * @return
     */
    @ApiOperation("收款列表")
    @GetMapping("/receipt/list")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "receiptId", value = "收款Id"),
            @ApiImplicitParam(required = false, name = "orderId", value = "订单Id"),
            @ApiImplicitParam(required = false, name = "orderNo", value = "订单编号"),
            @ApiImplicitParam(required = false, name = "userName", value = "机构名称"),
            @ApiImplicitParam(required = false, name = "payWay", value = "收款方式"),
            @ApiImplicitParam(required = false, name = "receiptProgress", value = "收款进度"),
            @ApiImplicitParam(required = false, name = "associationNum", value = "关联数量"),
            @ApiImplicitParam(required = false, name = "receiptStatus", value = "收款状态"),
            @ApiImplicitParam(required = false, name = "payType", value = "收款帐号"),
            @ApiImplicitParam(required = true, name = "organizeId", value = "组织Id"),
            @ApiImplicitParam(required = true, name = "pageNum", value = "当前页码"),
            @ApiImplicitParam(required = true, name = "pageSize", value = "每页大小")
    })
    public ResponseJson<PageInfo<ReceiptVo>> receiptList(ReceiptVo receiptVo,
                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return ResponseJson.success(cmDiscernReceiptService.findReceiptList(receiptVo, pageNum, pageSize));
    }

    @ApiOperation("收款详情")
    @ApiImplicitParam(required = true, name = "id", value = "收款Id")
    @GetMapping("/receipt/detail/{id}/{shopOrderId}")
    public ResponseJson<ReceiptDetail> receiptDetail(@PathVariable("id") Integer id,@PathVariable("shopOrderId") Integer shopOrderId) {
        return ResponseJson.success(cmDiscernReceiptService.findReceiptDetail(id,shopOrderId));
    }

    @ApiOperation("收款帐号列表")
    @GetMapping("/receipt/type")
    public ResponseJson<List<ReceiptPayType>> receiptPayType() {
        return ResponseJson.success(cmDiscernReceiptService.findPayType());
    }

    @ApiOperation("付款帐号列表")
    @GetMapping("/pay/type")
    public ResponseJson<List<ReceiptPayType>> payType() {
        return ResponseJson.success(cmDiscernReceiptService.payType());
    }
}
