package com.caimei.modules.order.entity;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.List;

/**
 * @description 主订单表
 */
@ApiModel("主订单实体")
@Data
public class NewOrder implements Serializable {

    private Integer orderId;

    private String orderNo;

    @ApiModelProperty("订单可见度")
    private Integer orderSeen;

    @ApiModelProperty("结算状态")
    private Integer settleStatus;

    @ApiModelProperty("邮费信息")
    private String PostageInfo;

    @ApiModelProperty("订单归属人userId")
    private Integer userId;

    private Integer organizeId;

    @ApiModelProperty("订单提交类型")
    private String orderSubmitType;

    @ApiModelProperty("订单状态0待确认,11待收待发,12待收部发,13待收全发,21部收待发," +
            "22部收部发,23部收全发,31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退")
    private String status;

    @ApiModelProperty("商品总金额")
    private Double productTotalFee;

    @ApiModelProperty("应收商品金额==应付合计（只需要付商品金额）")
    private Double shouldPayProduct;

    @ApiModelProperty("是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是")
    private String hasActProduct;

    @ApiModelProperty("订单总金额")
    private Double orderTotalFee;

    @ApiModelProperty("主订单总应付金额")
    private Double payableAmount;

    @ApiModelProperty("支付金额")
    private Double payTotalFee;

    @ApiModelProperty("余额支付")
    private Double balancePayFee;

    @ApiModelProperty("折扣金额")
    private Double discountFee;

    @ApiModelProperty("协销id")
    private Integer spId;

    @ApiModelProperty("订单备注(买家留言)")
    private String note;

    @ApiModelProperty("买家留言")
    private List<String> notes;

    private Integer clubId;

    @ApiModelProperty("订单提交时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String orderTime;

    @ApiModelProperty("订单确认时间")
    private String confirmTime;

    @ApiModelProperty("订单确认标志")
    private String confirmFlag;

    @ApiModelProperty("条款Id")
    private Integer clauseId;

    @ApiModelProperty("条款内容")
    private String clauseContent;

    @ApiModelProperty("条款名称")
    private String clauseName;

    @ApiModelProperty("免邮标志 运费：-1到付，0包邮，1需要运费")
    private String freePostFlag;

    @ApiModelProperty("运费：-1到付，0包邮，其他为具体运费")
    private Double freight;

    @ApiModelProperty("下单人")
    private Integer buyUserId;

    @ApiModelProperty("子订单")
    private List<NewShopOrder> newShopOrders;

    @ApiModelProperty("订单用户信息")
    private OrderUserInfo bpOrderUserinfo;

    private OrderInvoice orderInvoice;

    @ApiModelProperty("商品总数")
    private Integer productCount;

    @ApiModelProperty("赠送总数")
    private Integer presentCount;

    @ApiModelProperty("订单支付时间")
    private String payTime;

    @ApiModelProperty("是否已支付 未支付0 已支付1")
    private String payFlag;

    @ApiModelProperty("0 能线上 1 线下")
    private String onlinePayFlag;

    @ApiModelProperty("总优惠 自助下单活动优惠 协销下单price-折后单价")
    private Double preferential;

    @ApiModelProperty("订单取消原因")
    private String closeReason;

    @ApiModelProperty("是否开发票 0 不开 1 开发票")
    private String invoiceFlag;

    @ApiModelProperty("（收款买家）收款状态：1待收款、2部分收款、3已收款")
    private String receiptStatus;

    @ApiModelProperty("(付款供应商)付款状态：1待付款、2部分付款、3已付款")
    private String payStatus;

    @ApiModelProperty("发货状态：1待发货、2部分发货、3已发货")
    private String sendOutStatus;

    @ApiModelProperty("退货退款类型：0无退款、1部分退、2全部退")
    private String refundType;

    private String orderType;//    订单类型 0协销订单、 1普通订单、 2呵呵订单


    @ApiModelProperty("邮费订单标识 1是邮费订单  0不是邮费订单")
    private String postageOrderFlag = "0";

    @ApiModelProperty("主订单总已付金额")
    private Double paidAmount;


    @ApiModelProperty("供应商")
    private String shopName;

    @ApiModelProperty("收货人")
    private String receiver;

    @ApiModelProperty("买家")
    private String buyer;

    @ApiModelProperty("已收金额(需要包含运费,目前只存在普通订单存在运费)")
    private Double alreadyReceipt;

    @ApiModelProperty("佣金")
    private String brokerage;

    @ApiModelProperty("买家名称")
    private String buyerName;

    @ApiModelProperty("机构名称")
    private String clubName;

    @ApiModelProperty("手机")
    private String mobile;

    @ApiModelProperty("地址")
    private String address;

    /**
     * 订单提交时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String startTime;
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String endTime;

    @ApiModelProperty("总税费")
    private Double totalAddedValueTax;

    @ApiModelProperty("退款状态")
    private String orderRefundType;

    @ApiModelProperty("退款总额")
    private Double refundFee;

    @ApiModelProperty("退款1审核中，0不是审核中")
    private String returnedPurchaseStatus;

    @ApiModelProperty("0不可以申请，1可以申请（此标记只用于是否存在审核中或者审核不通过记录，存在记录则不可以申请）")
    private String applyReturnedPurchaseFlag;

    @ApiModelProperty("订单内的商品集合")
    private List<NewOrderProduct> orderProduct;

    @ApiModelProperty("子订单编号")
    private String shopOrderNo;

    @ApiModelProperty("有没有收款待审核的订单 , 有的1")
    private String toAudit;

    private String productName;

    /**
     * 当主订单为【待收款】或者为线下支付的【部分收款】状态时，才显示更改订单状态按钮
     * （排除交易全退订单）
     */
    @ApiModelProperty("展示字段：能否更改收款状态 1可以 0不可以")
    private Integer checkFlag;
    /**
     * 当主订单为【待收款】或者为线下支付的【部分收款】状态时，才显示更改订单状态按钮
     * （排除交易全退订单）
     */
    @ApiModelProperty("1WWW、2CRM、3APP[历史数据]、4客服、5外单、6小程序[采美,星范等]")
    private Integer orderSource;

    @ApiModelProperty("退款信息")
    private CmReturnedPurchase cmReturnedPurchase;


    private Integer organizeStoreId;    // 组织门店id
    private Integer shopOrderId;        // varchar(100) comment '子订单Id',
    private String shopOrderIds;        // varchar(100) comment '子订单Id',
    private Double promotionFullReduction;             //DECIMAL(20,2) DEFAULT 0.00  NULL   COMMENT '促销满减优惠' AFTER `discountFee`,
    private Integer mainSpId;            // bigint(6),
    private String clubScanTime;        // varchar(19),
    private String payWay;              // 支付方式,（协销订单可能会存在多种进账方式用,隔开）,
    private Integer payTypeId;           // int(11) comment '支付类型',
    private String cooFreeFlag;          // char(1) comment '库分期免息状态',
    private Integer cooFreeRate;         // int(3) comment '库分期分期费率',
    private Double cooFreeAmount;       // decimal(20,2) comment '库分期免息金额',
    private Integer promotionalGiftsCount; // INT(11) NULL   COMMENT '促销赠品总数' AFTER `presentCount
    private String splitFlag; // 订单能否拆分 1 为可拆分, 0为不可拆分
    private String closeTime;   // 订单关闭时间
    private Double discountAmount;  // 优惠金额：经理折扣+促销优惠+优惠券抵扣金额
    // ---------------- v5.0.0 新加
    private Double totalDiscount;//订单退款已退的经理折扣金额
    private Integer zeroCostFlag;//订单0成本标识
    private Integer userBeans;//采美豆抵扣运费数量(100:1)
    // -----------------  v5.0.0 end
    private Integer returnGoodsStutas;//能否退货  1 能 2 不能
    //-------------        虚拟字段   ----------------------------
    private Integer settleRecordNum;   //佣金结算数量
    private String unReceipt;//未收金额(需要包含运费,目前只存在普通订单存在运费)
    private String paid;//已付金额(不需要包含运费,等同于订单商品金额)
    private String unpaid;//未付金额(不需要包含运费,等同于订单商品金额)
    private String startConfirmTime;
    private String endConfirmTime;
    private String startRefundTime;
    private String endRefundTime;
    private String shopNameInfo;//
    private String spName;
    private Integer serviceProviderId;  //协销Id
    private Double settleAmount;   // 已结算金额
    private Integer splitCount;  //分账记录数

    // 促销
    private String searchflag;//0搜索，1导出
    private String[] shopNote;  //子订单备注   修改时用来记录一下的
    private String rebateOrder;  //是不是返佣订单 是的1,  不是的0
    private String[] ps;    //付款状态集合
    private Double shouldPayShopAmount; //付供应商总金额
    private Double returnValue;  //退款总金额
    private Boolean returnedFreightFlag;//机构运费退货标识
    private List<NewOrderProduct> newOrderProducts;  // 订单商品列表

    // 收款确认类型（1小额抹平确认，2大额抹平确认，3大额退款余额，4确认关联）
    private String confirmType;
    private Double orderReceivedAmount;//订单已收金额（此收款只计算线上线下收款记录payableAmount表金额）
    private Double returnBalanceAmount;// 退款余额金额
    private String secondHandOrderFlag;//二手商品订单标识  0非二手商品订单、 1二手商品订单
    private boolean receiptOrderFlag;// 判断订单中抹平的订单是否是和多个订单一起支付的，
    private Integer rechargeGoods; //1,缴纳订金;2,充值余额
    private String affirmPaymentFlag;// 二手订单确认付款供应商标识 0未确认，1已确认
    private String rebateFlag;//返佣订单标识，0非返佣订单，1返佣订单,2普通订单有返佣服务费
    private String rebateFee; // 返佣服务费
    private String associationType;//订单关联方式: 1手动 2自动
    private Double ableUserMoney; //当前下单用户可用余额
    private String organizeStoreName;//组织机构名称
    /**
     * 优惠券机构关联Id
     */
    private Integer clubCouponId;
    private Double couponAmount;    //优惠券抵扣金额
    private Double svipFullReduction;   //超级会员优惠
    private Integer collageFlag;    // 拼团订单标识：1拼团订单，0不是
    private Integer collageStatus;  // 拼团状态：1拼团中，2已拼成
    private Double reductionAmount;    //分享减免金额
    private Double reductionTouchPrice; //分享减免条件
    private Integer onlinfePayFlag;  //线上支付标记

    private Integer productCategory;//商品订单状态 1.普通订单 2.二手订单

    private Integer productCode;//订单商品是否是采美信息分帐号收款的线上支付子订单

    private Integer numbers;    // 总数
    private Double totalMoney;  // 总金额


    /**
     * 呵呵商城优惠券
     */
    private Integer receiveCouponId;
    private Integer delFlag;
}

