package com.caimei.modules.order.entity;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigInteger;
import java.util.List;

/**
 * @author lwt
 * @date 2018-07-24
 * @description 子订单表
 */
@ApiModel(description= "子订单")
@Data
public class NewShopOrder {

    @ApiModelProperty("邮费信息")
    private String postageInfo;
    @ApiModelProperty("是否包含冷链费 0不包含 1包含")
    private Integer isColdChina;
    @ApiModelProperty("展示字段：付款单详情所有子订单")
    private List<String> shopOrderNos;
    @ApiModelProperty("主订单金额")
    private Double payTotalFee;
    @ApiModelProperty("机构运费 0包邮 -1到付 -2仪器到付-产品包邮，均不是则为实付运费")
    private Double freight;
    @ApiModelProperty("主订单总佣金")
    private Double totalBrokerage;
    @ApiModelProperty("免邮标志 运费：-1到付，0包邮，1需要运费,-2仪器到付其它包邮")
    private String shopPostFlag;
    @ApiModelProperty("主订单收款金额")
    private Double receiptTotalFee;
    @ApiModelProperty("付款单的商品费, 子订单商品的成本价的和")
    private Double shopProductAmount;
    @ApiModelProperty("付供应商运费")
    private Double shopPostFee;
    @ApiModelProperty("(付款供应商)付款状态：1待付款、2部分付款、3已付款")
    private Integer payStatus;


    @ApiModelProperty("已付采美")
    private Double payCmAmount;
    @ApiModelProperty("应付采美")
    private Double shouldPayCmAmount;
    @ApiModelProperty("待付采美")
    private Double waitPayCmAmount;

    @ApiModelProperty("子订单待付金额")
    private Double restAmount;

    @ApiModelProperty("应付供应商 = 商品费 + 运费 + 税费")
    private Double shouldPayShopAmount;
    @ApiModelProperty("已付供应商")
    private Double payedShopAmount;
    @ApiModelProperty("待付供应商")
    private Double waitPayShop;

    @ApiModelProperty("虚拟字段：已付组织")
    private Double paidOrganize;
    @ApiModelProperty("虚拟字段：应付组织")
    private Double shouldPayOrganize;
    @ApiModelProperty("虚拟字段：待付组织")
    private Double waitPayOrganize;

    @ApiModelProperty("供应商名称")
    private String shopName;
    @ApiModelProperty("买家/客户")
    private String buyer;
    @ApiModelProperty("子订单收款状态：1待收款、2部分收款、3已收款")
    private String receiptStatus;
    @ApiModelProperty("主订单收款状态：1待收款、2部分收款、3已收款")
    private Integer orderReceiptStatus;


    @ApiModelProperty("结算状态 1未结算2部分结算3已结算")
    private Integer settleStatus;
    @ApiModelProperty("已结算供应商金额")
    private Double settleShopAmount;
    @ApiModelProperty("已结算采美金额")
    private Double settleCmAmount;
    @ApiModelProperty("已结算组织金额")
    private Double settleOrganizeAmount;
    @ApiModelProperty("应结算供应商金额")
    private Double shouldSettleShop;
    @ApiModelProperty("应结算采美金额")
    private Double shouldSettleCm;
    @ApiModelProperty("应结算组织金额")
    private Double shouldSettleOrganize;
    @ApiModelProperty("待结算供应商金额")
    private Double waitSettleShop;
    @ApiModelProperty("待结算采美金额")
    private Double waitSettleCm;
    @ApiModelProperty("待结算组织金额")
    private Double waitSettleOrganize;
    @ApiModelProperty("结算时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String settleTime;


    @ApiModelProperty("子订单Id")
    private Integer shopOrderId;
    @ApiModelProperty("子订单编号")
    private String shopOrderNo;
    @ApiModelProperty("0待确认,11待收待发,12待收部发,13待收全发,21部收待发,22部收部发,23部收全发,31已收待发,32已收部发,33已收全发,4交易完成,5订单完成,6已关闭,7交易全退")
    private String shopStatus;
    @ApiModelProperty("订单编号")
    private String orderNo;
    @ApiModelProperty("订单Id")
    private Integer orderId;
    @ApiModelProperty("下单时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String orderTime;


    @ApiModelProperty("子订单已收金额")
    private Double receiptAmount;
    @ApiModelProperty("子订单应收金额")
    private Double realPay;
    @ApiModelProperty("组织Id")
    private Integer organizeId;
    @ApiModelProperty("userId")
    private Integer userId;
    @ApiModelProperty("供应商Id")
    private Integer shopId;
    @ApiModelProperty("商品数量")
    private Integer itemCount;
    @ApiModelProperty("收货地址县区Id")
    private Integer townId;
    @ApiModelProperty("应收商品金额=应付合计（只需要付商品金额）")
    private Double shouldPayProduct;
    @ApiModelProperty("运费：-1到付，0包邮，其他为具体运费")
    private Double fee;
    @ApiModelProperty("余额支付时使用的金额")
    private Double accountAmount;
    @ApiModelProperty("总价")
    private Double totalAmount;
    @ApiModelProperty("实付金额")
    private Double needPayAmount;
    @ApiModelProperty("退款金额")
    private Double refundAmount;
    @ApiModelProperty("机构Id")
    private Integer clubId;
    @ApiModelProperty("协销Id")
    private Integer spId;

    @ApiModelProperty("是否可以退货 1可以退款/退货 0不可退款/退货")
    private Integer canRefundFlag;
    @ApiModelProperty("是否使用余额")
    private Integer useBalanceFlag;
    @ApiModelProperty("可以退还的采美豆个数")
    private Integer canRefundBeans;
    @ApiModelProperty("订单包邮时本该支付的运费")
    private Double freePostageFee;
    @ApiModelProperty("子订单佣金")
    private Double brokerage;
    @ApiModelProperty("后台删除状态 0正常，其他删除")
    private String delFlag;
    @ApiModelProperty("订单退款金额")
    private Double refundsAmount;
    @ApiModelProperty("订单状态标识，1:非退货退款订单、2：退货退款中、3退货退款完成")
    private String orderStatusFlag;
    @ApiModelProperty("订单商品list")
    private List<NewOrderProduct> newOrderProducts;
    @ApiModelProperty("订单商品的总优惠")
    private Double preferential;
    @ApiModelProperty("赠品数量")
    private Integer presentNum;
    @ApiModelProperty("税费")
    private Double totalAddedValueTax;
    @ApiModelProperty("收货时间")
    private String receiveGoodsTime;
    @ApiModelProperty("子订单备注信息")
    private String note;

    @ApiModelProperty("已经发货的商品数量")
    private Integer outStoreNum;
    @ApiModelProperty("子订单发货批次 默认值 = 0 即没过发货")
    private Integer outStoreTimes;
    @ApiModelProperty("发货状态：1待发货、2部分发货、3已发货")
    private String sendOutStatus;
    @ApiModelProperty("收货人")
    private String receiver;
    @ApiModelProperty("'0'=不是邮费商品子订单; '1'=是邮费商品子订单")
    private String postFeeCos = "0";
    @ApiModelProperty("机构名称")
    private String clubName;
    @ApiModelProperty("退款状态 1无退款,3有退款")
    private String refundStatus;
    @ApiModelProperty("支付凭证")
    List<Voucher> vouchers;
    /**
     *  付款列表展示数据
     */
    @ApiModelProperty("是否处于给供应商状态中 0不是,1是")
    private Integer paying;
    @ApiModelProperty("是否处于供应商退款状态中  0不是,1是")
    private String refunding;
    @ApiModelProperty("商品金额/商品费")
    private Double productAmount;
    @ApiModelProperty("已付金额")
    private Double payed;
    @ApiModelProperty("分帐号")
    private String splitCode;
    @ApiModelProperty("分账时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private String splitTime;
}
