package com.caimei.modules.order.service.impl;

import com.caimei.modules.order.dao.CmOrderRemarkDao;
import com.caimei.modules.order.entity.CmOrderRemark;
import com.caimei.modules.order.entity.OssFile;
import com.caimei.modules.order.entity.RemarkImage;
import com.caimei.modules.order.service.CmOrderRemarkService;
import com.caimei.modules.order.util.OssService;
import com.caimei.modules.shiro.entity.CmMallAdminUser;
import com.caimei.utils.OSSUtils;
import com.caimei.utils.ResponseJson;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.shiro.SecurityUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;

@Slf4j
@Service
public class CmOrderRemarkServiceImpl implements CmOrderRemarkService {

    @Resource
    private CmOrderRemarkDao cmOrderRemarkDao;
    @Resource
    private OssService ossService;
    @Resource
    private OSSUtils ossUtils;

    @Override
    public List<CmOrderRemark> findList(Integer shopOrderID) {
        List<CmOrderRemark> list = cmOrderRemarkDao.findList(shopOrderID);
        list.forEach(l -> {
            l.setImages(cmOrderRemarkDao.findImages(l.getId()));
            l.setOssFiles(cmOrderRemarkDao.findFiles(l.getId()));
        });
        return list;
    }

    @Override
    public void insertRemark(CmOrderRemark remark) {
        CmMallAdminUser user = (CmMallAdminUser) SecurityUtils.getSubject().getPrincipal();
        remark.setCreateBy(user.getId());
        cmOrderRemarkDao.insert(remark);
        List<String> images = remark.getImages();
        List<OssFile> ossFiles = remark.getOssFiles();
        if (null != images && images.size() > 0) {
            images.forEach(i -> {
                RemarkImage remarkImage = new RemarkImage();
                remarkImage.setRemarkImage(i);
                remarkImage.setRemarkId(remark.getId());
                remarkImage.setOrderId(remark.getOrderId());
                cmOrderRemarkDao.insertImage(remarkImage);
            });
        }
        if (null != ossFiles && ossFiles.size() > 0) {
            ossFiles.forEach(o -> {
                o.setRemarkId(remark.getId());
                o.setOrderId(remark.getOrderId());
                cmOrderRemarkDao.insertFile(o);
            });
        }
    }

    @Override
    public ResponseJson<Map<String, Object>> upload(MultipartFile file, Integer orderId) {
        Map<String, Object> map = new HashMap<>();
        String fileAllName = file.getOriginalFilename();
        String fileType = fileAllName.substring(fileAllName.lastIndexOf(".") + 1);
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        String filePath = uuid + "." + fileType;
        String contentType = ossUtils.getContentType(fileAllName);
        try {
            //保存本地
            File fil = ossUtils.ossUpload(file);
            //判断文件的唯一性,转换成16进制md5值
            String md5Hex = DigestUtils.md5Hex(new FileInputStream(fil));
            OssFile remarkFile = cmOrderRemarkDao.findFileByMd5(md5Hex);
            String url = "";
            if (remarkFile == null) {
                log.info("默认路径>>>" + fil.getAbsolutePath());
                //上传oss
                url = ossUtils.ossUpload(filePath, fil, contentType);
            } else {
                filePath = remarkFile.getOssName();
            }
            //删除本地文件
            ossUtils.deleteFile(fil);
            //保存关联关系
            OssFile orderRemarkFile = new OssFile();
            orderRemarkFile.setOrderId(orderId);
            orderRemarkFile.setName(fileAllName);
            orderRemarkFile.setOssName(filePath);
            orderRemarkFile.setMd5Hex(md5Hex);
            map.put("url", url);
            map.put("ossFiles", orderRemarkFile);
            return ResponseJson.success(map);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("备注文件上传异常！！！");
            return ResponseJson.error("备注文件上传异常");
        }
    }

    @Override
    public void downFile(Integer fileId, HttpServletRequest request, HttpServletResponse response) throws IOException {
        OssFile remarkFile = cmOrderRemarkDao.findRemarkById(fileId);
        String fileName = remarkFile.getName();
        ossUtils.downFile(remarkFile.getOssName(), fileName);
        ossService.download(request, response, fileName);
    }
}
