package com.caimei.modules.order.service.impl;

import com.caimei.modules.club.dao.CmUserDao;
import com.caimei.modules.club.entity.CmUser;
import com.caimei.modules.order.dao.*;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.CmReturnedPurchaseService;
import com.caimei.modules.order.service.CmUserBalanceRecordService;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.modules.order.service.NewShopOrderService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.AppUtils;
import com.caimei.utils.MathUtil;
import com.caimei.utils.NewOrderStatus;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;

@Slf4j
@Service
public class CmReturnedPurchaseServiceImpl implements CmReturnedPurchaseService {

    @Value("${malladmin.domain}")
    private String domain;

    @Resource
    private CmReturnedPurchaseDao cmReturnedPurchaseDao;
    @Resource
    private CmReturnedPurchaseProductDao cmReturnedPurchaseProductDao;
    @Resource
    private NewShopOrderService newShopOrderService;
    @Resource
    private NewOrderService orderService;
    @Resource
    private NewOrderDao orderDao;
    @Resource
    private CmUserBalanceRecordService cmUserBalanceRecordService;
    @Resource
    private OrderUtils orderUtil;
    @Resource
    private PayShopDao payShopDao;
    @Resource
    private NewShopOrderDao shopOrderDao;
    @Resource
    private CmUserDao cmUserDao;
    @Resource
    private NewOrderProductDao newOrderProductDao;


    @Override
    public List<CmReturnedPurchase> getReturnedPurchaseList(CmReturnedPurchase cmReturnedPurchase) {
        List<CmReturnedPurchase> dataList = new ArrayList<CmReturnedPurchase>();
        dataList = cmReturnedPurchaseDao.findList(cmReturnedPurchase);
        if (null != dataList && dataList.size() > 0) {
            for (CmReturnedPurchase bean : dataList) {
                CmReturnedPurchaseProduct cmReturnedPurchaseProduct = new CmReturnedPurchaseProduct();
                cmReturnedPurchaseProduct.setReturnedId(bean.getId());
                cmReturnedPurchaseProduct.setShopOrderId(bean.getShopOrderId());
                List<CmReturnedPurchaseProduct> list = cmReturnedPurchaseProductDao.findList(cmReturnedPurchaseProduct);
                for (CmReturnedPurchaseProduct item : list) {
                    //设置全路径
                    item.setMainImage(AppUtils.getImageURL("product", item.getMainImage(), 0, domain));
                    //计算商品的未出库和已取消和已发货数量
                    Integer returnedPurchaseProductNum = 0;
                    Integer cancelProductNum = 0;
                    //获取退货退款记录
                    CmReturnedPurchaseProduct purchaseProduct = new CmReturnedPurchaseProduct();
                    purchaseProduct.setShopOrderId(item.getShopOrderId());
                    purchaseProduct.setProductId(item.getProductId());
                    List<CmReturnedPurchaseProduct> cmReturnedPurchaseProductList = cmReturnedPurchaseProductDao.getCmReturnedPurchaseProductList(purchaseProduct);
                    if (null != cmReturnedPurchaseProductList && cmReturnedPurchaseProductList.size() > 0) {
                        for (CmReturnedPurchaseProduct c : cmReturnedPurchaseProductList) {
                            returnedPurchaseProductNum += c.getActualReturnedNum();
                            cancelProductNum += c.getActualCancelNum();
                        }
                    }
                    item.setReturnedPurchaseProductNum(returnedPurchaseProductNum);
                    item.setCancelProductNum(cancelProductNum);
                    //判断订单商品是否是首次退货退款
                    Boolean isFirstReturn = false;
                    List<CmReturnedPurchaseProduct> temporaryList = cmReturnedPurchaseProductDao.findListByReturnTime(bean.getOrderId(), item.getShopOrderId(), item.getOrderProductId(), bean.getReturnTime());
                    if (null == temporaryList || temporaryList.size() == 0) {
                        isFirstReturn = true;
                    }
                    item.setFirstReturn(isFirstReturn);
                }

                //遍历List根据子订单分组
                Map map = new HashMap<String, List>();
                for (CmReturnedPurchaseProduct bean1 : list) {
                    if (map.containsKey(bean1.getShopOrderId())) {
                        List data = (List) map.get(bean1.getShopOrderId());
                        data.add(bean1);
                    } else {
                        List data = new ArrayList();
                        data.add(bean1);
                        map.put(bean1.getShopOrderId(), data);
                    }
                }
                //遍历map 把刚刚按照子订单排序的map取出来放到对象中
                List<ShopOrderReturned> shopOrderReturnedList = new ArrayList<ShopOrderReturned>();
                Iterator<Map.Entry<String, List<CmReturnedPurchaseProduct>>> entries = map.entrySet().iterator();
                while (entries.hasNext()) {
                    ShopOrderReturned shopOrderReturned = new ShopOrderReturned();
                    Map.Entry<String, List<CmReturnedPurchaseProduct>> entry = entries.next();
                    if (entry.getValue().size() > 0) {
                        shopOrderReturned.setCmReturnedPurchaseProductList(entry.getValue());
                        CmReturnedPurchaseProduct item = entry.getValue().get(0);
                        shopOrderReturned.setShopName(item.getShopName());
                        shopOrderReturned.setShopId(item.getShopId());
                        shopOrderReturned.setShopOrderId(item.getShopOrderId());
                        shopOrderReturned.setReturnedReceiver(item.getReturnedReceiver());
                        NewShopOrder shopOrder = newShopOrderService.findByShopOrderId(item.getShopOrderId());
                        if (null != shopOrder) {
                            shopOrderReturned.setProductAmount(shopOrder.getProductAmount());
                            shopOrderReturned.setPayStatus(shopOrder.getPayStatus());
                            shopOrderReturned.setSendOutStatus(shopOrder.getSendOutStatus());
                            shopOrderReturned.setShopOrderNo(shopOrder.getShopOrderNo());
                        }
                    }
                    shopOrderReturnedList.add(shopOrderReturned);
                }

                bean.setShopOrderReturnedList(shopOrderReturnedList);
            }
        }
        return dataList;
    }

    @Override
    public List<CmReturnedPurchase> findList(CmReturnedPurchase cmReturnedPurchase) {
        return cmReturnedPurchaseDao.findList(cmReturnedPurchase);
    }

    @Override
    public CmReturnedPurchase get(Integer id) {
        return cmReturnedPurchaseDao.get(id);
    }

    @Override
    public void toReturnRecord(NewOrder newOrder, CmReturnedPurchase cmReturnedPurchase) {
        orderUtil.setReceiverVal(newOrder);
//        List<NewShopOrder> shopOrders = cmReturnedPurchaseDao.findReturnShopOrder(cmReturnedPurchase.getId());
        newOrder.getNewShopOrders().forEach(s -> {
            List<NewOrderProduct> orderProducts = cmReturnedPurchaseProductDao.findProductByShopOrder(s.getShopOrderId(), cmReturnedPurchase.getId());
            s.setNewOrderProducts(orderProducts);
        });
//        newOrder.setNewShopOrders(shopOrders);
    }

    @Override
    public List<NewOrder> getReturnDetailList(Integer orderId) {
        NewOrder newOrder = orderService.get(orderId.toString());
        List<Integer> ids = cmReturnedPurchaseDao.findReturnIds(orderId);
        ArrayList<NewOrder> newOrders = new ArrayList<>();
        ids.forEach(i -> {
            CmReturnedPurchase cmReturnedPurchase = new CmReturnedPurchase();
            toReturnRecord(newOrder, cmReturnedPurchase);
            newOrders.add(newOrder);
        });
        return newOrders;
    }

    @Override
    public Map<String, Object> getReturnAndPay(Integer shopOrderId) {
        Map<String, Object> map = new HashMap<>();

        NewShopOrder shopOrder = shopOrderDao.findByShopOrderId(shopOrderId);
        orderUtil.getShouldPay(shopOrder);
        orderUtil.setShopPostInfo(shopOrder);
        List<OrderPayShopRecord> pays = payShopDao.findPayShopRecordsByShopOrder(shopOrderId);
        List<ReturnRecord> returns = cmReturnedPurchaseDao.findRecordByShopOrder(shopOrderId);
        map.put("shopOrder", shopOrder);
        map.put("pays", pays);
        map.put("returns", returns);
        return map;
    }

    @Transactional(readOnly = false)
    @Override
    public void revokeApply(CmReturnedPurchase cmReturnedPurchase) {
        try {
            //1.撤销退款退货表审核通过信息
            cmReturnedPurchase.setConfirmReturnTime(null);
            cmReturnedPurchase.setReviewImage1(null);
            cmReturnedPurchase.setReviewImage2(null);
            cmReturnedPurchase.setReviewImage3(null);
            cmReturnedPurchase.setReviewImage4(null);
            cmReturnedPurchase.setReviewImage5(null);
            cmReturnedPurchase.setStatus("1");
            cmReturnedPurchase.setReviewUserId(null);
            cmReturnedPurchase.setReviewRemarks(null);
            cmReturnedPurchaseDao.updateRevokeApply(cmReturnedPurchase);
            NewOrder newOrder = orderService.get(cmReturnedPurchase.getOrderId().toString());
            NewShopOrder newShopOrder = shopOrderDao.get(cmReturnedPurchase.getShopOrderId());
            //2.查询除本次撤销记录外存在的其他的有效记录
            CmReturnedPurchase item = new CmReturnedPurchase();
            item.setStatus("2");
//            item.setOrderId(cmReturnedPurchase.getOrderId());
            item.setShopOrderId(cmReturnedPurchase.getShopOrderId());
            List<CmReturnedPurchase> returnedPurchaseRecord = cmReturnedPurchaseDao.findReturnedPurchaseRecord(item);
            if (CollectionUtils.isNotEmpty(returnedPurchaseRecord)) {
                //存在退款记录则本次撤销后订单状态为部分退款
                //部分退货
                newOrder.setRefundType("1");
                String status = newShopOrder.getShopStatus();
                if (StringUtils.equals("5", status)) {//如果之前为交易全退，撤销后则变为
                    if (StringUtils.equals("1", newShopOrder.getReceiptStatus())) {//（收款买家）收款状态：1待收款、2部分收款、3已收款
                        newShopOrder.setShopStatus(NewOrderStatus.DAISHOU_BUFA.getCode());//待收部发
                    }
                    if (StringUtils.equals("2", newShopOrder.getReceiptStatus())) {
                        newShopOrder.setShopStatus(NewOrderStatus.BUSHOU_BUFA.getCode());//部收部发
                    }
                    if (StringUtils.equals("3", newShopOrder.getReceiptStatus())) {
                        newShopOrder.setShopStatus(NewOrderStatus.YISHOU_BUFA.getCode());//已收部发
                    }
                }
            } else {
                //不存在退款记录则本次退款为未退款
                String status = newShopOrder.getShopStatus();
                newOrder.setRefundType("");
                if (StringUtils.equals("5", status)) {//如果之前为交易全退，撤销后则变为
                    if (StringUtils.equals("1", newShopOrder.getReceiptStatus())) {//（收款买家）收款状态：1待收款、2部分收款、3已收款
                        newShopOrder.setShopStatus(NewOrderStatus.DAISHOU_DAIFA.getCode());//待收待发
                    }
                    if (StringUtils.equals("2", newShopOrder.getReceiptStatus())) {
                        newShopOrder.setShopStatus(NewOrderStatus.BUSHOU_DAIFA.getCode());//部收待发
                    }
                    if (StringUtils.equals("3", newShopOrder.getReceiptStatus())) {
                        newShopOrder.setShopStatus(NewOrderStatus.YISHOU_DAIFA.getCode());//已收待发
                    }
                }
            }
            newShopOrder.setRefundStatus("1");//无退款
            //更新订单信息
            orderDao.update(newOrder);
            shopOrderDao.update(newShopOrder);

            //更新用户余额总金额
            Integer userID = newOrder.getUserId();
            CmUser cmUser = cmUserDao.get(userID);
            Double refundBalanceFee = cmReturnedPurchase.getRefundBalanceFee();
            cmUser.setUserId(userID);

            CmUserBalanceRecord balanceRecord = cmUserBalanceRecordService.getByCmUserBalanceRecord(new CmUserBalanceRecord().setReturnedId(cmReturnedPurchase.getId()));
            //充值或缴纳订金订单
            if (balanceRecord != null) {
                balanceRecord.setRemark("已撤销退款");
                cmUser.setUserMoney(cmUser.getUserMoney() + cmReturnedPurchase.getRefundFee());
            } else {
                cmUser.setAbleUserMoney(cmUser.getAbleUserMoney() - refundBalanceFee);
                cmUser.setUserMoney(cmUser.getUserMoney() - refundBalanceFee);
            }

            //撤销审核通过的余额记录
            if (refundBalanceFee > 0) {//存在余额退款
                //撤销余额收支记录
                balanceRecord.setUserId(newOrder.getUserId());
//                balanceRecord.setOrderId(newOrder.getOrderId());
                balanceRecord.setShopOrderId(cmReturnedPurchase.getShopOrderId());
                balanceRecord.setReturnedId(cmReturnedPurchase.getId());
                balanceRecord.setRemark("撤销退款，余额记录无效-后台");
                balanceRecord.setDelFlag("1");
            }
            cmUserDao.update(cmUser);
            if(balanceRecord!=null){
                cmUserBalanceRecordService.updateCmUserBalanceRecord(balanceRecord);
            }
//            //撤销退款采美豆
//            Integer userBeans = Integer.valueOf(cmUser.getUserBeans());
//            Integer num = cmUserbeanshistoryDao.findByBeansType(newOrder.getOrderID());
//            if (num != null && num > 0) {
//                userBeans = userBeans + num;
//            }
//            if (newOrder.getUserBeans() != null && newOrder.getUserBeans() > 0) {
//                userBeans = userBeans - newOrder.getUserBeans();
//            }
//            cmUserbeanshistoryDao.updateBeansHistoryByReturned(newOrder.getOrderID());
//            cmUserDao.updateUserBeans(newOrder.getUserID(), userBeans);
//            //撤销优惠券退款
//            if (newOrder.getCouponAmount() > 0) {
//                cmCouponClubDao.deleteByReturnedId(cmReturnedPurchase.getId());
//            }
            //修改子订单信息
            updateShopOrderAmount(newOrder.getOrderId(),cmReturnedPurchase.getShopOrderId(),returnedPurchaseRecord);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void updateShopOrderAmount(Integer orderId,Integer shopOrderId,List<CmReturnedPurchase> returnedPurchaseRecord) {
        //修改子订单付供应商金额
        List<NewShopOrder> shopOrders = shopOrderDao.findListByOrderId(orderId);
        shopOrders.removeIf(s -> 998 == s.getShopId());
        for (NewShopOrder shopOrder : shopOrders) {
            if(shopOrderId!=null&&shopOrderId!=shopOrder.getShopOrderId()){
                continue;
            }
            List<NewOrderProduct> orderProducts = newOrderProductDao.findListByShopOrderId(shopOrder.getShopOrderId());
            BigDecimal totalCostFee = BigDecimal.ZERO;
            BigDecimal shopTaxFee = BigDecimal.ZERO;
            for (NewOrderProduct orderProduct : orderProducts) {
                Integer count = newOrderProductDao.CountReturnedPurchaseProduct(shopOrder.getShopOrderId(), orderProduct.getOrderProductId());
                count = count == null ? 0 : count;
                Integer number = orderProduct.getNum() + orderProduct.getPresentNum() - count;
                totalCostFee = totalCostFee.add(MathUtil.mul(number, orderProduct.getCostPrice()));
                shopTaxFee = shopTaxFee.add(MathUtil.mul(number, orderProduct.getSingleShouldPayTotalTax()));
            }
            // 商品费
            shopOrder.setShopProductAmount(totalCostFee.doubleValue());
            // 应付税费
            shopOrder.setTotalAddedValueTax(shopTaxFee.doubleValue());
            if (null == shopOrder.getShopPostFee()) {
                shopOrder.setShopPostFee(0D);
            }
            // 付供应商
            double shouldPayShopAmount = shopOrder.getShopProductAmount() + shopOrder.getShopPostFee() + shopOrder.getTotalAddedValueTax();
            shopOrder.setShouldPayShopAmount(shouldPayShopAmount);
            if (CollectionUtils.isNotEmpty(returnedPurchaseRecord)) {
                //存在退款记录则本次撤销后订单状态为部分退款
                //部分退货
                shopOrder.setOrderStatusFlag("2");
                String status = shopOrder.getShopStatus();
                if (StringUtils.equals("7", status)) {//如果之前为交易全退，撤销后则变为
                    if (StringUtils.equals("1", shopOrder.getReceiptStatus())) {//（收款买家）收款状态：1待收款、2部分收款、3已收款
                        shopOrder.setShopStatus(NewOrderStatus.DAISHOU_BUFA.getCode());//待收部发
                    }
                    if (StringUtils.equals("2", shopOrder.getReceiptStatus())) {
                        shopOrder.setShopStatus(NewOrderStatus.BUSHOU_BUFA.getCode());//部收部发
                    }
                    if (StringUtils.equals("3", shopOrder.getReceiptStatus())) {
                        shopOrder.setShopStatus(NewOrderStatus.YISHOU_BUFA.getCode());//已收部发
                    }
                }
            } else {
                //不存在退款记录则本次退款为未退款
                String status = shopOrder.getShopStatus();
                shopOrder.setOrderStatusFlag("2");
                if (StringUtils.equals("7", status)) {//如果之前为交易全退，撤销后则变为
                    if (StringUtils.equals("1", shopOrder.getReceiptStatus())) {//（收款买家）收款状态：1待收款、2部分收款、3已收款
                        shopOrder.setShopStatus(NewOrderStatus.DAISHOU_DAIFA.getCode());//待收待发
                    }
                    if (StringUtils.equals("2", shopOrder.getReceiptStatus())) {
                        shopOrder.setShopStatus(NewOrderStatus.BUSHOU_DAIFA.getCode());//部收待发
                    }
                    if (StringUtils.equals("3", shopOrder.getReceiptStatus())) {
                        shopOrder.setShopStatus(NewOrderStatus.YISHOU_DAIFA.getCode());//已收待发
                    }
                }
            }
            shopOrderDao.update(shopOrder);
        }

    }
}
