package com.caimei.modules.order.service.impl;

import com.caimei.modules.order.dao.*;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.NewOrderService;
import com.caimei.modules.order.service.OrderDeliverService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.utils.ResponseJson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @author zzj
 */
@Slf4j
@Service
public class OrderDeliverServiceImpl implements OrderDeliverService {

    @Resource
    private NewOrderDao orderDao;
    @Resource
    private OrderUtils orderUtil;
    @Resource
    private NewOrderProductDao orderProductDao;
    @Resource
    private LogisticsBatchDao logisticsBatchDao;
    @Resource
    private LogisticsRecordDao recordDao;
    @Resource
    private LogisticsInformationDao informationDao;
    @Resource
    private NewOrderDao newOrderDao;

    @Override
    public NewOrder toDeliver(Integer orderId) {
        NewOrder newOrder = orderDao.get(orderId.toString());
        orderUtil.setReceiverVal(newOrder);
        List<NewOrderProduct> orderProducts = orderProductDao.findListByOrderId(orderId);
        orderProducts.forEach(o -> {
            o.setNotOutStore(o.getNotOutStore() - orderProductDao.getActualCancelNum(o.getOrderProductId()));
        });
        newOrder.setOrderProduct(orderProducts);
        return newOrder;
    }

    @Override
    @Transactional(readOnly = false, rollbackFor = Exception.class)
    public ResponseJson confirmDeliver(DeliverDto dto) {
        List<NewOrderProduct> orderProducts = dto.getOrderProducts();
        if (null == orderProducts || orderProducts.size() == 0) {
            return ResponseJson.error("发货商品不能为空！");
        }
        // 设置子订单/主订单发货状态
        Boolean status = orderUtil.setDeliverStatus(orderProducts);
        if (!status) {
            return ResponseJson.error("发货数量异常！");
        }
        Integer orderId = orderProducts.get(0).getOrderId();
        //新增发货批次记录
        LogisticsBatch batch = new LogisticsBatch();
        batch.setOrderId(orderId);
        batch.setMailer(2);
        batch.setUpdateDate(new Date());
        batch.setDeliveryTime(new Date());
        batch.setRemark(dto.getRemark());
        StringBuilder stringBuilder = new StringBuilder();
        if (null != dto.getImages() && dto.getImages().size() > 0) {
            dto.getImages().forEach(i -> stringBuilder.append(i).append("##"));
        }
        batch.setRemarkImage(stringBuilder.toString());
        logisticsBatchDao.insert(batch);

        Map<Integer, Integer> shopSetNum = new HashMap<>();
        orderProducts.forEach(p -> {
            boolean flag = shopSetNum.containsKey(p.getShopOrderId());
            if (flag) {
                Integer num = shopSetNum.get(p.getShopOrderId());
                shopSetNum.put(p.getShopOrderId(), p.getDeliveryNum() + num);
            }
            LogisticsRecord record = new LogisticsRecord();
            record.setLogisticsBatchId(batch.getId());
            record.setShopOrderId(p.getShopOrderId());
            record.setOrderId(orderId);
            record.setOrderProductId(p.getOrderProductId());
            record.setBuyNum(p.getNum());
            record.setNum(p.getDeliveryNum());
            record.setProductId(p.getProductId());
            record.setProductName(p.getName());
            record.setImage(p.getProductImage());
            recordDao.insert(record);
        });

        List<Transportation> transportations = dto.getTransportations();
        transportations.forEach(t -> {
            t.getLogisticsNu().forEach(c -> {
                LogisticsInformation formation = new LogisticsInformation();
                formation.setLogisticsBatchId(batch.getId());
                formation.setType(4);
                formation.setOrderId(orderId);
                formation.setNu(c.getNum());
                /**
                 * 快递单当前的状态（0：在途，即货物处于运输过程中；
                 * 1：揽件，货物已由快递公司揽收并且产生了第一条跟踪信息；
                 * 2：疑难，货物寄送过程出了问题；
                 * 3：签收，收件人已签收；
                 * 4：退签，即货物由于用户拒签、超区等原因退回，而且发件人已经签收；
                 * 5：派件，即快递正在进行同城派件；
                 * 6：退回，货物正处于退回发件人的途中；）
                 */
                formation.setState("1");
                formation.setRemarks(dto.getRemark());
                //物流公司名称
                formation.setLogisticsCompanyName(t.getLogisticsCompanyName());
                formation.setLogisticsCompanyCode(t.getLogisticsCompanyCode());
                formation.setUpdateDate(new Date());
                informationDao.insert(formation);
            });
        });
        return ResponseJson.success();
    }

    @Override
    public ResponseJson<List<LogisticsInfoVo>> findDetails(Integer orderId ,Integer shopOrderId) {
        List<LogisticsInfoVo> result = new ArrayList<>();
        //查询这个子订单所有的发货批次记录
        List<LogisticsBatch> lbs = logisticsBatchDao.findBatch(orderId, null).stream().filter(remark -> remark.getShopOrderId() .equals(shopOrderId)).collect(Collectors.toList());
        //过滤子订单
        NewOrder newOrder = newOrderDao.get(orderId.toString());
        orderUtil.setOrderValue(newOrder);
        List<NewShopOrder>  newShopOrders= newOrder.getNewShopOrders().stream().filter(newShopOrder -> newShopOrder.getShopOrderId().equals(shopOrderId)).collect(Collectors.toList());
        newOrder.setNewShopOrders(newShopOrders);
        if(lbs!=null&&lbs.size()>0){
            lbs.get(0).setNewOrder(newOrder);
        }

        for (LogisticsBatch l : lbs) {
            LogisticsInfoVo liv = new LogisticsInfoVo();
            // 每一批的发货商品记录
            List<LogisticsRecord> lrs = recordDao.findByLogisticsBatchId(l.getId());
            // 每一批的发货物流记录
            List<LogisticsInformation> lis = informationDao.findByLogisticsBatchId(l.getId());
            liv.setLogisticsBatch(l);
            liv.setLogisticsInformationList(lis);
            liv.setLogisticsRecordList(lrs);
            result.add(liv);
        }
        return ResponseJson.success(result);
    }

    @Override
    public ResponseJson<List<CompanyVo>> getLogisticsCompany(String value) {
        return ResponseJson.success(logisticsBatchDao.findCompany(value));
    }
}
