package com.caimei.modules.order.service.impl;

import com.caimei.modules.order.constant.Constant;
import com.caimei.modules.order.dao.NewOrderDao;
import com.caimei.modules.order.dao.NewShopOrderDao;
import com.caimei.modules.order.dao.PayShopDao;
import com.caimei.modules.order.entity.*;
import com.caimei.modules.order.service.PayShopService;
import com.caimei.modules.order.util.OrderUtils;
import com.caimei.modules.shiro.entity.CmMallAdminUser;
import com.caimei.utils.*;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;


/**
 * @author zzj
 */
@Slf4j
@Service
public class PayShopServiceImpl implements PayShopService {

    @Resource
    private RedisService redisService;
    @Resource
    private PayShopDao payShopDao;
    @Resource
    private NewShopOrderDao newShopOrderDao;
    @Resource
    private NewOrderDao newOrderDao;
    @Resource
    private OrderUtils orderUtils;


    @Override
    public PageInfo<OrderPayShop> findPayList(OrderPayShop orderPayShop, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<OrderPayShop> payShops = payShopDao.findPayShops(orderPayShop);
        payShops.forEach(p -> {
            p.setShopOrderNos(payShopDao.findShopOrderNosByPayShop(p.getPayShopId()));
            p.setOrderNo(payShopDao.findOrderNos(p.getPayShopId()));
        });
        return new PageInfo(payShops);
    }

    @Override
    public OrderPayShopDetail getPayShopDetail(Integer id) {
        OrderPayShopDetail detail = payShopDao.findPayShopDetail(id);
        List<NewShopOrder> shopOrders = new ArrayList<>();
        double totalPayShop = 0d;
        double totalPayCm = 0d;
        if (1 == detail.getStatus()) {
            // 已付款待付金额活动计算
            shopOrders = payShopDao.findPayShopOrders(detail.getPayShopId());
            shopOrders.forEach(s -> {
                orderUtils.getShouldPay(s);
            });
            totalPayShop = shopOrders.stream().mapToDouble(NewShopOrder::getPayedShopAmount).sum();
            totalPayCm = shopOrders.stream().mapToDouble(NewShopOrder::getPayCmAmount).sum();
        } else {
            // 未付款待付显示为本次申请金额
            shopOrders = payShopDao.findPayShopOrderWaitPay(detail.getPayShopId());
            shopOrders.forEach(s -> {
                if (s.getShopPostFee() > 0) {
                    s.setShouldPayShopAmount(s.getShouldPayShopAmount() + s.getShopPostFee());
                }
                List<NewOrderProduct> products = payShopDao.findPayOrderProducts(s.getShopOrderId());
                s.setNewOrderProducts(products);
            });
            // 待审核总付 = 本次申请
            totalPayShop = shopOrders.stream().mapToDouble(NewShopOrder::getWaitPayShop).sum();
            // 待审核总付 =  本次申请
            totalPayCm = shopOrders.stream().mapToDouble(NewShopOrder::getWaitPayCmAmount).sum();
        }
        detail.setShopOrders(shopOrders);
        orderUtils.setPayShopPostInfo(detail);
        // 付供应商
        detail.setTotalAmount(totalPayShop);
        // 付采美
        detail.setPayCmTotalAmount(totalPayCm);
        return detail;
    }

    @Transactional(readOnly = false, rollbackFor = Exception.class)
    @Override
    public ResponseJson checkPass(Integer id, Integer payType, Integer passCode, String reason) throws Exception {
        OrderPayShop orderPayShop = new OrderPayShop();
        CmMallAdminUser user = (CmMallAdminUser) SecurityUtils.getSubject().getPrincipal();
        orderPayShop.setReviewer(user.getId());
        orderPayShop.setReviewTime(DateUtils.getDateTime());
        orderPayShop.setPayShopId(id);
        orderPayShop.setReason(reason);
        orderPayShop.setPayType(payType);
        if (1 == passCode) {
            // 通过审核
            List<OrderPayShopRecord> records = payShopDao.findPayShopRecords(id);
            Integer orderId = 0;
            //修改每一条子订单的已付款金额
            for (OrderPayShopRecord record : records) {
                NewShopOrder shopOrder = payShopDao.findShopOrderByPayShop(id);
                orderUtils.getShouldPay(shopOrder);
                orderId = shopOrder.getOrderId();
                // 本次付款金额 + 付采美 + 原来已经付过的金额
                Double payedFee = record.getPayAmount() + record.getPayCmAmount() + shopOrder.getPayedShopAmount() + shopOrder.getPayCmAmount();
                // 总共要付的金额
                BigDecimal shouldPayFee = MathUtil.add(shopOrder.getShouldPayShopAmount(), shopOrder.getShouldPayCmAmount());
                if (MathUtil.compare(payedFee, shouldPayFee) == -1) {
                    // (本次付款金额 + 付采美 + 原来已经付过的金额)  < 总共要付的金额
                    shopOrder.setPayStatus(2);
                } else if (MathUtil.compare(payedFee, shouldPayFee) == 0) {
                    // 本次付款金额 + 原来已经付过的金额  近似于 总共要付的金额相等
                    shopOrder.setPayStatus(3);
                } else {
                    log.info("已付款金额:" + shopOrder.getPayedShopAmount() + "本次付款金额:" + record.getPayAmount() + "付供应商:" + shouldPayFee + "付款id：" + id);
                    log.info("check ----> 付款金额异常, 付款金额大于待付款金额");
                    return ResponseJson.error("付款单审核金额异常, 付款金额大于待付款金额");
                }
                shopOrder.setPayedShopAmount(shopOrder.getPayedShopAmount() == null ? 0D : shopOrder.getPayedShopAmount() + record.getPayAmount());
                shopOrder.setPayCmAmount(shopOrder.getPayCmAmount() == null ? 0D : shopOrder.getPayCmAmount() + record.getPayCmAmount());
                //退出付款进行状态
                shopOrder.setPaying(0);
                newShopOrderDao.update(shopOrder);
                record.setStatus(1);
                record.setPayType(payType);
                payShopDao.updatePayRecord(record);
            }
            //查询未付
            Integer count = newShopOrderDao.findUnPaidShopOrder(orderId);
            Integer status = count > 0 ? 2 : 3;
            newOrderDao.updatePayStatus(status, orderId);

            orderPayShop.setPayTime(DateUtils.getDateTime());
            orderPayShop.setStatus(1);
            payShopDao.updatePayShop(orderPayShop);
            return ResponseJson.success();
        } else {
            orderPayShop.setStatus(2);
            payShopDao.updatePayShop(orderPayShop);
            //付款记录 修改状态
            payShopDao.updatePayRecordById(2, id);
            //子订单退出付款状态
            newShopOrderDao.outPaying(id);
            return ResponseJson.success();
        }
    }

    @Transactional(readOnly = false, rollbackFor = Exception.class)
    @Override
    public ResponseJson<Integer> applyPay(OrderPayShopDetail payShop) {
        CmMallAdminUser user = (CmMallAdminUser) SecurityUtils.getSubject().getPrincipal();
        payShop.setApplicant(user.getId());
        List<NewShopOrder> shopOrders = payShop.getShopOrders();
        if (null == payShop.getPayShopId()) {
            //新增
            payShop.setApplyTime(DateUtils.getDateTime());
            payShop.setStatus(0);
            payShopDao.insertPayShop(payShop);

            payShopDao.updateShopAccount(payShop.getShopId(), payShop.getBankAccount(), payShop.getBankAccountName(), payShop.getBankName());
            payShopDao.updateCmShopAccount(payShop.getCmBankAccount(), payShop.getCmBankAccountName(), payShop.getCmBankName());
            shopOrders.forEach(s -> {
                OrderPayShopRecord record = new OrderPayShopRecord();
                record.setShopId(payShop.getShopId());
                record.setShopOrderId(s.getShopOrderId());
                record.setShopOrderNo(s.getShopOrderNo());
                record.setPayAmount(s.getWaitPayShop());
                record.setPayCmAmount(s.getWaitPayCmAmount());
                record.setPayType(payShop.getPayType());
                record.setPayShopId(payShop.getPayShopId());
                record.setStatus(0);
                payShopDao.insertPayRecord(record);
                //子订单发起付款申请以后 就不能同时再次发起付款申请了  直到审核操作通过或者取消
                // 同时修改付供应商运费
                payShopDao.updatePaying(s.getShopOrderId(), payShop.getShopPostFee());
            });

            // 运费付款单作用不明，暂时取消生成

        } else {
            if (null == payShop.getOperateMode()) {
                //修改
                if (1 == payShop.getStatus()) {
                    return ResponseJson.error("该付款单已被审核，无需再申请付款");
                }
                if (2 == payShop.getStatus()) {
                    //付款记录 修改状态
                    payShopDao.updatePayRecordById(2, payShop.getPayShopId());
                    //子订单退出付款状态
                    newShopOrderDao.inPaying(payShop.getPayShopId());
                    payShop.setReviewer(null);
                    payShop.setReviewTime(null);
                    payShop.setReason(null);
                    payShop.setStatus(0);
                }
                shopOrders.forEach(s -> {
//                    if (s.getShopPostFee() > 0) {
//                        // 修改时，以数据库中的应付为准，应付 - 上一次的
//                        Double shopPostFee = payShopDao.findShopPostFee(s.getShopOrderId());
//                        // 应付为+了供应商运费的金额，应付 - 数据库应付 = 上一次的供应商运费
//                        // 待付 = 待付 - 上一次的供应商运费
//                        s.setWaitPayShop(s.getWaitPayShop() - shopPostFee);
//                    }
                    payShopDao.updateRecordById(s.getShopOrderId(), s.getWaitPayShop(), s.getWaitPayCmAmount(), payShop.getPayShopId());
                });
            }
            payShopDao.updatePayShopDetail(payShop);
        }
        // 如果供应商运费不为0 , 修改子订单供应商运费
        payShop.getShopOrders().forEach(s -> payShopDao.updateShopPost(s.getShopOrderId(), s.getShopPostFee()));
        return ResponseJson.success(payShop.getPayShopId());
    }

    @Transactional(readOnly = false)
    @Override
    public ResponseJson cancelPay(Integer id) {
        OrderPayShopDetail payShopDetail = payShopDao.findPayShopDetail(id);
        if (1 == payShopDetail.getStatus()) {
            return ResponseJson.error("该付款单已被审核，不能取消");
        }
        newShopOrderDao.outPaying(id);
        payShopDao.deleteRecords(id);
        payShopDao.deletePayShop(id);
        return ResponseJson.success();
    }

    @Transactional(readOnly = false, rollbackFor = Exception.class)
    @Override
    public ResponseJson splitShopOrders(List<Integer> shopOrderIds) {
        Integer xsfzmds = (Integer) redisService.get("OGFZMDS");
        //验锁
        if (null != xsfzmds && xsfzmds > 0) {
            return ResponseJson.error("请勿短时间内重复分账，等待支付公司返回分账结果！");
        }
        log.info("【手动分账开始】>>>>>>>>>>手动分账");
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String currentTime = format.format(calendar.getTime());
        shopOrderIds.forEach(s -> {
            // 查询未分账已支付收款
            List<CmReceiptOrderRelation> orderRelations = newOrderDao.getUndividedPaidReceipt(currentTime, s);
            if (null != orderRelations && orderRelations.size() > 0) {
                for (CmReceiptOrderRelation orderRelation : orderRelations) {
                    log.info("【分账】>>>>>>>>>>子订单id:" + s + "进入分账");
                    // 收款对应的订单信息
                    NewShopOrder shopOrder = newOrderDao.getShopOrderListByOrderId(orderRelation.getShopOrderId());
                    orderUtils.getShouldPay(shopOrder);
                    PayParamBo payParam = new PayParamBo();
                    //支付金额
                    payParam.setAllPay(orderRelation.getAssociateAmount());
                    payParam.setPayType(orderRelation.getPayType().toString());
                    List<SplitAccountPo> splitBillDetail = new ArrayList<>();
                    /**
                     *  todo
                     *  供应商承担手续费,在计算过程中无需处理,按正常比例分配分组织佣金和分采美佣金,
                     *  佣金分配完成后剩余金额则为供应商成本分账金额 - 供应商承担手续费,不在cm_split_account中做成本扣减体现
                     *  后续有对账需求再做处理
                     */
                    setSplitAccountDetail(shopOrder, payParam, splitBillDetail);
                    HashMap<String, BigDecimal> sbm = new HashMap<>();
                    for (SplitAccountPo splitAccountPo : splitBillDetail) {
                        String subUserNo = splitAccountPo.getSubUserNo();
                        // 计算当前商户号总分账金额
                        if (sbm.containsKey(subUserNo)) {
                            BigDecimal v = MathUtil.add(sbm.get(subUserNo), splitAccountPo.getSplitAccount());
                            sbm.put(subUserNo, v);
                        } else {
                            sbm.put(subUserNo, BigDecimal.valueOf(splitAccountPo.getSplitAccount()));
                        }
                        // splitcode相同的收款的时候已经是成本已分帐
                        if (subUserNo.equals(shopOrder.getSplitCode())) {
                            splitAccountPo.setMbOrderId(orderRelation.getMbOrderId());
                            splitAccountPo.setOrderRequestNo(orderRelation.getOrderRequestNo());
                            // 供应商自己收款，此部分金额留在自己商户号，作为成本分账
                            splitAccountPo.setPayStatus(1);
                            // 保存分账详情
                            newOrderDao.insertSplitAccount(splitAccountPo);
                        }
                    }
                    ArrayList<AccountPayOrder.AccountPayOrderExt.SplitBillRule> splitBillRules = new ArrayList<>();
                    sbm.forEach((key, value) -> {
                        /**
                         *   不是自己的splitcode分走，是自己的不动
                         *   把成本之外的金额加入splitRule分账参数,延续上面逻辑,供应商自己的成本由自己的商户号收款,
                         *   自己成本部分不需要加入分账参数,仅需在else中做业务表参数处理
                         */
                        if (!key.equals(shopOrder.getSplitCode())) {
                            AccountPayOrder.AccountPayOrderExt.SplitBillRule splitBillRule = new AccountPayOrder.AccountPayOrderExt.SplitBillRule();
                            /**
                             *  由于供应商承担付款手续费，只用参与真实分账部分处理0.1%分账手续费,
                             *  不在setSplitAccountDetail计算过程中扣除分账手续费
                             */
                            // 分账计算的时候未处理手续费，总额处理手续费   x + x * 0.1% = value
                            BigDecimal div = MathUtil.div(value, 1.001, 2);
                            if (MathUtil.sub(value, div).doubleValue() < 0.01) {
                                // 如果手续费小于最低手续费，取0.01
                                div = MathUtil.round(MathUtil.sub(value, 0.01), 2);
                            }
                            splitBillRule.setSplitBillAmount(div);
                            splitBillRule.setSplitBillMerchantNo(key);
                            splitBillRules.add(splitBillRule);
                        } else {
                            // 成本分账处理
                            // 子订单的splitcode和分账对象的splitcode一样的是留在账户里的钱，实际在付款完成后，付款状态就为已付款，分账状态就为已分账
                            // 为维持业务完整性，保持原来的分账流程不变
                            List<SplitAccountPo> splitAccountList = newOrderDao.getSplitAccountList(orderRelation.getMbOrderId());
                            if (splitAccountList != null && splitAccountList.size() > 0) {
                                Integer orderId = splitAccountList.get(0).getOrderId();
                                List<NewShopOrder> shopOrderList = newOrderDao.getShopOrderByOrderId(orderId);
                                Integer shopOrderId = null;
                                String shopOrderNo = "";
                                for (SplitAccountPo account : splitAccountList) {
                                    // 本次付供应商金额(分账金额)
                                    Double splitAmount = account.getSplitAccount();
                                    Integer shopId = account.getShopId();
                                    for (NewShopOrder shop : shopOrderList) {
                                        if (shopId.equals(shop.getShopId())) {
                                            shopOrderId = shop.getShopOrderId();
                                            shopOrderNo = shop.getShopOrderNo();
                                            // 已付供应商金额
                                            Double paidAmount = newOrderDao.getPaidShopAmount(shopOrderId);
                                            Double paidShop = MathUtil.add(paidAmount, splitAmount).doubleValue();
                                            shopOrder.setPayedShopAmount(paidShop);
                                            //todo
//                                            if (MathUtil.compare(shopOrder.getShouldPayShopAmount(), paidShop) == 0) {
                                            shopOrder.setPayStatus(3);
//                                            } else {
//                                                shopOrder.setPayStatus(2);
//                                            }
                                            // 修改子订单付款状态及付款金额
                                            newOrderDao.updateShopOrderByPayStatus(shopOrderId, paidShop, shopOrder.getPayStatus());
                                        }
                                    }
                                    SimpleDateFormat format2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                    String currentTime2 = format2.format(new Date());
                                    // 保存付款单表
                                    OrderPayShopDetail payShop = new OrderPayShopDetail();
                                    payShop.setShopId(shopId);
                                    payShop.setName("线上支付分账");
                                    payShop.setTotalAmount(splitAmount);
                                    payShop.setPayType(6);
                                    payShop.setStatus(1);
                                    payShop.setApplyTime(currentTime2);
                                    payShop.setReviewTime(currentTime2);
                                    payShop.setPayTime(currentTime2);
                                    payShopDao.insertPayShop(payShop);
                                    // 保存 付供应商记录
                                    OrderPayShopRecord shopRecord = new OrderPayShopRecord();
                                    shopRecord.setShopId(shopId);
                                    shopRecord.setShopOrderId(shopOrderId);
                                    shopRecord.setShopOrderNo(shopOrderNo);
                                    shopRecord.setPayAmount(splitAmount);
                                    shopRecord.setPayType(6);
                                    shopRecord.setPayShopId(payShop.getPayShopId());
                                    shopRecord.setStatus(1);
                                    payShopDao.insertPayRecordForNow(shopRecord);
                                }
                                // 子订单是否全部付款
                                Integer count = newShopOrderDao.findUnPaidShopOrder(orderRelation.getOrderId());
                                // 修改主订单付款状态
                                Integer status = count > 0 ? 2 : 3;
                                newOrderDao.updatePayStatus(status, shopOrder.getOrderId());
                            }
                        }
                    });
                    /**
                     *  在上述else代码逻辑中已完成成本的业务表参数处理，在分账详情中排除成本,剩余佣金部分在分账结束收到
                     *  成功返回码时处理（成本不参与分账）
                     */
                    splitBillDetail.removeIf(sp -> sp.getSubUserNo().equals(shopOrder.getSplitCode()));
                    if (null != splitBillRules && splitBillRules.size() > 0) {
                        orderUtils.splitAccount(splitBillRules, orderRelation.getOrderRequestNo(), orderRelation.getMbOrderId(), shopOrder, splitBillDetail);
                    } else {
                        // 没有佣金分账，全是成本，本笔交易已分帐
                        newOrderDao.updateBySplitStatus(orderRelation.getMbOrderId());
                        // 没有佣金分账也解锁
                        redisService.remove("OGFZMDS");
                    }
                }
            }
        });
        return ResponseJson.success();
    }

    @Override
    public ResponseJson settleShopOrders(OrderPayShopDetail payShop) {
        List<NewShopOrder> shopOrders = payShop.getShopOrders();
        AtomicInteger flag = new AtomicInteger();
        shopOrders.forEach(o -> {
            Calendar instance = Calendar.getInstance();
            int weekIdx = instance.get(Calendar.DAY_OF_WEEK) - 1;
            String form = new SimpleDateFormat("HH:mm:ss").format(new Date());
            Integer sub = Integer.valueOf(form.substring(0, 2));
            //当前时间周五下午到周日都返回1不能结算,其他时间看够不够24小时
            if ((5 == weekIdx && sub > 14) || 6 == weekIdx || 7 == weekIdx) {
                flag.set(1);
                return;
            } else {
                Integer val = payShopDao.getSplitTime(o.getShopOrderId());
                if (val <= 0) {
                    flag.set(1);
                    return;
                }
            }
        });
        if (1 == flag.get()) {
            return ResponseJson.error("当前时间为周五下午到周日，或分账完成时间不足24小时，无法结算！");
        }
        String format = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss.SSS").format(new Date());
        String substring = format.substring(20);
        shopOrders.forEach(s -> {
            try {
                orderUtils.settleOrder("JSCB" + s.getShopOrderId() + substring, s.getWaitSettleShop(), s.getSplitCode(), s.getShopOrderId(), 1);
                orderUtils.settleOrder("JSCMYJ" + s.getShopOrderId() + substring, s.getWaitSettleCm(), Constant.CUSTOMERNUM2, s.getShopOrderId(), 3);
                orderUtils.settleOrder("JSYJ" + s.getShopOrderId() + substring, s.getWaitSettleOrganize(), Constant.LHLGCUSTOMERNUM, s.getShopOrderId(), 5);
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
        NewShopOrder shopOrder = newShopOrderDao.findColdChainShopOrder(shopOrders.get(0).getOrderId());
        // 联合丽格冷链费
        if (null != shopOrder && shopOrder.getShopPostFee() >= 0) {
            try {
                orderUtils.settleOrder("JSLLF" + shopOrder.getShopOrderId() + substring, shopOrder.getShopPostFee(), Constant.LHLGCUSTOMERNUM, shopOrder.getShopOrderId(), 6);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return ResponseJson.success();
    }


    @Override
    public PageInfo<NewShopOrder> findPayOrderList(OrderPayShop orderPayShop, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<NewShopOrder> shopOrders = newShopOrderDao.findPayShopOrderList(orderPayShop);
        shopOrders.forEach(s -> {
            orderUtils.getShouldPay(s);
            orderUtils.setShopPostInfo(s);
        });
        return new PageInfo(shopOrders);
    }

    @Override
    public ResponseJson<OrderPayShopDetail> applyPayList(List<Integer> shopOrderIds) {
        if (null != shopOrderIds && shopOrderIds.size() > 0) {
            OrderPayShopDetail detail = new OrderPayShopDetail();
            List<NewShopOrder> shopOrders = payShopDao.findPayShopOrderByShopOrderIds(shopOrderIds);
            shopOrders.forEach(s -> orderUtils.getShouldPay(s));
            detail.setShopOrders(shopOrders);
            orderUtils.setPayShopPostInfo(detail);
            detail.setShopId(shopOrders.get(0).getShopId());
            detail.setShopName(shopOrders.get(0).getShopName());
            detail.setClubName(shopOrders.get(0).getClubName());
            return ResponseJson.success(detail);
        }
        return ResponseJson.error("未选择子订单！");
    }

    @Override
    public ResponseJson<PageInfo<NewOrder>> orderPayInfoList(OrderPayShop orderPayShop, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<NewOrder> orderList = newOrderDao.findListByPayShop(orderPayShop);
        orderList.forEach(o -> orderUtils.setOrderValue(o));
        return ResponseJson.success(new PageInfo<NewOrder>(orderList));
    }

    @Override
    public ResponseJson<Map<String, Object>> findSplitRecord(Integer orderId) {
        Map<String, Object> val = new HashMap<>(2);
        NewOrder newOrder = newOrderDao.get(orderId.toString());
        orderUtils.setReceiverVal(newOrder);
        orderUtils.setRefundVal(newOrder);
        val.put("order", newOrder);
        List<SplitRecord> splitRecord = orderUtils.getSplitRecord(newOrder);
        val.put("splitRecord", splitRecord);
        return ResponseJson.success(val);
    }

    @Override
    public ResponseJson<Map<String, Object>> findReceiptRecord(Integer orderId,Integer shopOrderId) {
        Map<String, Object> val = new HashMap<>(3);
        //过滤子订单
        NewOrder newOrder = newOrderDao.get(orderId.toString());
        orderUtils.setReceiverVal(newOrder);
        orderUtils.setOrderValue(newOrder);

        List<NewShopOrder>  newShopOrders= newOrder.getNewShopOrders().stream().filter(newShopOrder -> newShopOrder.getShopOrderId().equals(shopOrderId)).collect(Collectors.toList());
        newOrder.setNewShopOrders(newShopOrders);
        if(newOrder.getNewShopOrders()!=null&&newOrder.getNewShopOrders().size()>0){
            newOrder.getNewShopOrders().get(0).setPostageInfo(newOrder.getPostageInfo());
        }
        val.put("order", newOrder);
        newOrder.setShopOrderId(shopOrderId);
        List<ReceiptVo> receiptVos = orderUtils.getReceiptRecord(newOrder);
        val.put("receiptList", receiptVos);
        List<OrderPayShopRecord> payRecords = orderUtils.getPayRecords(newOrder);
        val.put("payList", payRecords);
        return ResponseJson.success(val);
    }

    @Transactional(readOnly = false, rollbackFor = Exception.class)
    @Override
    public ResponseJson backPay(Integer id) {
        //回滚付款单审核通过的所有操作
        List<OrderPayShopRecord> records = payShopDao.findPayShopRecords(id);
        List<Integer> status = new ArrayList<>();
        AtomicReference<Integer> orderId = new AtomicReference<>();
        for (OrderPayShopRecord record : records) {
            NewShopOrder shopOrder = newShopOrderDao.get(record.getShopOrderId());
            orderId.set(shopOrder.getOrderId());
            shopOrder.setPayedShopAmount(shopOrder.getPayedShopAmount() - record.getPayAmount());
            if (shopOrder.getPayedShopAmount() == 0D) {
                // 已付 - 撤回 = 0
                shopOrder.setPayStatus(1);
            } else if (shopOrder.getPayedShopAmount() > 0D) {
                // 已付 - 撤回 > 0
                shopOrder.setPayStatus(2);
            } else {
                log.info("rollBackApply ----> 付款金额异常, 付款金额大于待付款金额");
                return ResponseJson.error("付款金额异常, 退款金额大于已付款金额");
            }
            //进入付款进行状态
            shopOrder.setPaying(1);
            newShopOrderDao.update(shopOrder);
            status.add(shopOrder.getPayStatus());
        }
        payShopDao.backPayRecord(id);
        Integer orderStatus = status.contains(2) ? 2 : 1;
        payShopDao.backPayShop(id);
        newOrderDao.updatePayStatus(orderStatus, orderId.get());
        return ResponseJson.success();
    }

    @Override
    public ResponseJson<Map<String, Object>> findSettleRecord(Integer orderId) {
        Map<String, Object> val = new HashMap<>(2);
        NewOrder newOrder = newOrderDao.get(orderId.toString());
        orderUtils.setReceiverVal(newOrder);
        orderUtils.setRefundVal(newOrder);
        orderUtils.setSettleStatus(newOrder);
        val.put("order", newOrder);
        List<SettleRecord> settleRecord = orderUtils.getSettleRecord(newOrder);
        val.put("settleRecord", settleRecord);
        return ResponseJson.success(val);
    }


    public void setSplitAccountDetail(NewShopOrder shopOrder, PayParamBo payParam, List<SplitAccountPo> list) {
        // 待分账总金额 = 本次支付金额,单位/元
        double splitAmount = payParam.getAllPay();
        // 商品数据
        List<NewOrderProduct> orderProductList = newOrderDao.getOrderProductByShopOrderId(shopOrder.getShopOrderId());
        // 子订单付组织
        AtomicReference<BigDecimal> totalOrganize = new AtomicReference<>();
        // 子订单商品成本
        AtomicReference<BigDecimal> totalCostPrice = new AtomicReference<>();
        for (NewOrderProduct orderProduct : orderProductList) {
            // 总价 * 供应商百分比 = 成本分账金额
            double costPrice = MathUtil.mul(orderProduct.getTotalAmount(), MathUtil.div(orderProduct.getShopPercent(), 100)).doubleValue();
            // 供应商承担手续费 ，减去手续费
            String payType = payParam.getPayType();
            double charge = 0d;
            if ("17".equals(payType)) {
                // 17 个人网银 0.2% 最低0.1
                charge = Math.max(MathUtil.mul(splitAmount, 0.002, 2).doubleValue(), 0.1);
            } else if ("12".equals(payType)) {
                // 企业网银单笔10元
                charge = 10;
            } else if ("13".equals(payType) || "15".equals(payType)) {
                // 13PC-微信支付 15 微信小程序支付 0.65% 最低0.01
                charge = Math.max(MathUtil.mul(splitAmount, 0.0065, 2).doubleValue(), 0.01);
            } else if ("14".equals(payType)) {
                //  14PC-支付宝 0.25% 最低0.01
                charge = Math.max(MathUtil.mul(splitAmount, 0.0025, 2).doubleValue(), 0.01);
            } else if ("29".equals(payType)) {
                // 29快捷支付借记卡  0.3% 最低0.1
                charge = Math.max(MathUtil.mul(splitAmount, 0.003, 2).doubleValue(), 0.1);
            } else if ("30".equals(payType)) {
                //  30快捷支付贷记卡 0.6%  最低0.1
                charge = Math.max(MathUtil.mul(splitAmount, 0.006, 2).doubleValue(), 0.1);
            }
            costPrice = costPrice - charge;
            splitAmount = splitAmount - charge;
            totalCostPrice.set(MathUtil.add(costPrice, totalCostPrice.get()));
            double organize = MathUtil.mul(orderProduct.getTotalAmount(), MathUtil.div(orderProduct.getOrganizePercent(), 100), 2).doubleValue();
            totalOrganize.set(MathUtil.add(organize, totalOrganize.get()));
            /**
             * todo
             * 出于兼容性考虑，且多次支付手续费不同，分账方式为循环每次支付记录，
             * 当前业务满足子订单支付完成才进行分账，可以循环子订单统计支付金额，
             * 进行一次性分账，在当前模式下如果用户分多次支付，精度损失问题较为常见。
             */
            // 判断是否支付过
            Double paidAmount = newOrderDao.getOrderProductPaidAmount(orderProduct.getOrderProductId());
            // 支付过金额大于0
            if (null != paidAmount && MathUtil.compare(paidAmount, 0) > 0) {
                //已付>0，成本-已付
                costPrice = MathUtil.sub(costPrice, paidAmount).doubleValue();
            }
            // 待分账金额>成本
            if (MathUtil.compare(splitAmount, costPrice) > 0) {
                // 待分账金额-成本
                splitAmount = MathUtil.sub(splitAmount, costPrice).doubleValue();
            } else {
                costPrice = splitAmount;
                splitAmount = 0.00;
            }
            if (costPrice > 0) {
                SplitAccountPo splitAccount = new SplitAccountPo();
                splitAccount.setShopOrderId(shopOrder.getShopOrderId());
                splitAccount.setOrderId(shopOrder.getOrderId());
                splitAccount.setOrderProductId(orderProduct.getOrderProductId());
                splitAccount.setShopId(orderProduct.getShopId());
                splitAccount.setSplitAccount(costPrice);
                splitAccount.setProductType(1);
                // 该商品设置了商户号
                splitAccount.setType(4);
                splitAccount.setSubUserNo(shopOrder.getSplitCode());
                log.info("成本分账参数------------->" + splitAccount.toString());
                list.add(splitAccount);
            }
            if (MathUtil.compare(splitAmount, 0) == 0) {
                break;
            }
        }
        /**
         *  还有钱就付供应商运费
         *  线上暂无供应商运费，此处逻辑空置
         */

        // 如果还有钱则为佣金,分到组织
        if (MathUtil.compare(splitAmount, 0) > 0) {
            // 剩余金额 - totalOrganize
            Double amount = Math.max(totalOrganize.get().doubleValue(), 0.01);
            splitAmount = MathUtil.sub(splitAmount, amount).doubleValue();
            SplitAccountPo splitAccount = new SplitAccountPo();
            splitAccount.setOrderId(shopOrder.getOrderId());
            splitAccount.setShopOrderId(shopOrder.getShopOrderId());
            splitAccount.setSplitAccount(amount);
            splitAccount.setProductType(5);
            splitAccount.setType(5);
            /**
             * 目前写死为联合丽格，后续有其他组织再做改动
             */
            splitAccount.setSubUserNo(Constant.LHLGCUSTOMERNUM);
            log.info("组织佣金分账参数------------->" + splitAccount);
            list.add(splitAccount);
        }

        // 如果还有钱则为佣金,分到采美
        if (MathUtil.compare(splitAmount, 0) > 0) {
            Double amount = Math.max(splitAmount, 0.01);
            SplitAccountPo splitAccount = new SplitAccountPo();
            splitAccount.setOrderId(shopOrder.getOrderId());
            splitAccount.setShopOrderId(shopOrder.getShopOrderId());
            splitAccount.setSplitAccount(amount);
            splitAccount.setProductType(3);
            splitAccount.setType(5);
            splitAccount.setSubUserNo(Constant.CUSTOMERNUM2);
            log.info("采美佣金分账参数------------->" + splitAccount);
            list.add(splitAccount);
        }
    }

}
