package com.caimei.modules.shiro.service.impl;

import com.caimei.modules.club.entity.Page;
import com.caimei.modules.shiro.dao.MallUserDao;
import com.caimei.modules.shiro.entity.CmMallAdminUser;
import com.caimei.modules.shiro.service.MallUserService;
import com.caimei.utils.MD5Util;
import com.caimei.utils.ResponseJson;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * Description
 *
 * @author : Charles
 * @date : 2023/5/25
 */
@Service
public class MallUserServiceImpl implements MallUserService {

    @Autowired private MallUserDao mallUserDao;

    /**
     * 后台用户信息集合
     *
     * @param cmMallAdminUser
     * @return
     */
    @Override
    public ResponseJson<Page<CmMallAdminUser>> getMallUserInfo(CmMallAdminUser cmMallAdminUser, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum,pageSize);
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<CmMallAdminUser> mallUser = mallUserDao.getMallUserInfo(cmMallAdminUser);
        mallUser.forEach(u -> {
            u.setNewAddTime(dateFormat.format(u.getAddTime()));
            u.setNewUpdateTime(dateFormat.format(u.getUpdateTime()));
        });
        Page<CmMallAdminUser> page = new Page<>(mallUser);
        return ResponseJson.success(page);
    }

    /**
     * 修改用户上下线状态
     * @param status
     * @param id
     */
    @Override
    public ResponseJson updateUser(Integer status,  Integer id) {
        mallUserDao.updateUser(status, id);
        return ResponseJson.success();
    }

    /**
     * 获取用户信息
     *
     * @param id
     * @return
     */
    @Override
    public CmMallAdminUser getMallUserById(Integer id) {
        CmMallAdminUser mallUserById = mallUserDao.getMallUserById(id);
        // 确认密码
        mallUserById.setOldPassword(mallUserById.getPassword());
        return mallUserById;
    }

    /**
     * 保存用户信息
     *
     * @param cmMallAdminUser
     */
    @Override
    public ResponseJson saveUser(CmMallAdminUser cmMallAdminUser) {

        // 密码添加  随机盐
        UUID uuid = UUID.randomUUID();
        String[] split = uuid.toString().split("-");
        String newPassword = cmMallAdminUser.getPassword() + split[0];
        String s = MD5Util.md5(newPassword);
        cmMallAdminUser.setPassword(s);
        cmMallAdminUser.setSalt(split[0]);
        cmMallAdminUser.setPersonnelType(2);
        cmMallAdminUser.setUpdateTime(new Date());
        if (null == cmMallAdminUser.getId()) {
            // 添加组织后台用户信息
            cmMallAdminUser.setAddTime(new Date());
            cmMallAdminUser.setDelFlag("0");

            mallUserDao.insertMallUser(cmMallAdminUser);
        } else {
            // 修改组织后台用户信息
            mallUserDao.updateMallUser(cmMallAdminUser);
        }
        return ResponseJson.success();
    }

    /**
     * 重置密码
     * @throws Exception
     */
    @Override
    public ResponseJson repeatPassword(String password, Integer id){
        CmMallAdminUser cmMallUser = new CmMallAdminUser();
        // 密码添加  随机盐
        UUID uuid = UUID.randomUUID();
        String[] split = uuid.toString().split("-");
        String newPassword = password + split[0];
        String s = MD5Util.md5(newPassword);
        cmMallUser.setId(id);
        cmMallUser.setPassword(s);
        cmMallUser.setSalt(split[0]);
        cmMallUser.setUpdateTime(new Date());
        // 修改组织后台用户信息
        mallUserDao.updateMallUser(cmMallUser);
        return ResponseJson.success();
    }
}
