package com.caimei.utils;


import com.alibaba.fastjson.JSONObject;
import com.caimei.modules.fastDFS.FastDFSClient;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.annotation.Resource;
import java.io.*;
import java.util.Hashtable;
import java.util.Random;
import java.util.UUID;

@Slf4j
@Controller
@RequestMapping(value = "/mall/uploadFile")
public class ImageUploadUtils {
    protected static final Logger logger = LoggerFactory.getLogger(ImageUploadUtils.class);

    @Value("${malladmin.imageDomain}")
    private String imageDomain;

    @Resource
    private FastDFSClient client;

    private static final long serialVersionUID = 1L;

    // Ajax获取图片文件,与控件type=File中的name属性一样
    private File fileMaterial;
    // Ajax获取图片相对路径名称,相应的name属性名称+FileName
    private String dirName;
    private Hashtable<String, String> jsonResult;
    private static Hashtable<String, String> dirNameTableHashtable = new Hashtable<String, String>();

    @PostMapping("/image/upload")
    public ResponseJson<String> multiImageUpload(MultipartFile[] file, @RequestHeader HttpHeaders headers) {
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("【图片上传】 X-CLIENT-IP : " + ip);
        if (file != null && file.length > 0) {
            try {
                String fileUrl = null;
                for (MultipartFile fileItem : file) {
                    String url = saveFile(fileItem);
                    fileUrl = imageDomain + "/" + url;
                }
                log.info("【图片上传】>>>>>>>>>>>>>>>>上传成功:" + fileUrl);
                return ResponseJson.success("上传成功", fileUrl);
            } catch (IOException e) {
                log.error("【图片上传】>>>>>>>>>>>>>>>>上传失败:" + e);
                return ResponseJson.error("图片上传失败！",null);
            }
        } else {
            return ResponseJson.error("请传入图片！",null);
        }
    }


    private String saveFile(MultipartFile file) throws IOException {
        String originalFilename = file.getOriginalFilename();
        String randomStr = UUID.randomUUID().toString();
        assert originalFilename != null;
        int index = originalFilename.lastIndexOf(".");
        String name = originalFilename.substring(index);
        // 图片暂存本地服务器路径
        String filePath = "/mnt/newdatadrive/data/runtime/cloud-instance/server-tools/tempImage/";
//        if ("dev".equals(profile)){
//            filePath = "D:\\";
//        }
        filePath += randomStr + name;
        // 临时图片
        File tempFile = new File(filePath);
        file.transferTo(tempFile);
        log.info("【图片上传】>>>>>>>>>>>>>>>>图片临时路径:" + filePath);
        String fileUrl = client.uploadFile(filePath);
        // 删除临时图片
        boolean delete = tempFile.delete();
        log.info("【图片上传】>>>>>>>>>>>>>>>>删除临时图片:" + delete);
        return fileUrl;
    }

    @ResponseBody
    @RequestMapping(value = "/imageUpload", method = RequestMethod.POST)
    public JsonModel imageUpload(@RequestBody String imgStr) throws FileNotFoundException {
        JsonModel jsonModel = JsonModel.newInstance();
        logger.info("--------------------imgStr:"+imgStr);
        if(!StringUtils.isEmpty(imgStr)){
            JSONObject jsStr = JSONObject.parseObject(imgStr);
            imgStr = (String)jsStr.get("imgStr");
            logger.info("-----imgStr:"+imgStr);
            if(imgStr.contains(",")){
                String[] split = imgStr.split("base64,");
                imgStr = imgStr.substring(22, imgStr.length());//从base64截取到最后一位，因为直接使用split存在丢失问题。
                String s = split[0];//data:image/png;base64
                String imgStrType = ".jpg";
                if(s.contains("image/")){
                    imgStrType="." + s.substring(s.indexOf("image/")+1, s.indexOf(";"));
                }
//                imgStr = split[1];
                logger.info("-----imgStr1:"+imgStr);
                try {
                    String filepath = GenerateImage(imgStr,imgStrType);//解析编码
                    String result = client.uploadFile(filepath);
                    if (!StringUtils.isEmpty(result)) {
                        result = imageDomain + "/" + result;
                        logger.info(">>>>>>>>>>>>>>>>图片上传成功:" + result);
                        jsonModel.success(result);
                    } else {
                        logger.info(">>>>>>>>>>>>>>>>图片上传失败:");
                        jsonModel.error("图片上传失败");
                    }
                } catch (Exception e) {
                    logger.info("图片上传异常:" + e.getMessage());
                    jsonModel.error("图片上传异常");
                }

            }else{
                jsonModel.error("图片格式错误");
            }
        }else{
            jsonModel.error("图片为空");
        }
        return jsonModel;
    }

    @ResponseBody
    @RequestMapping(value = "/testImageUpload")
    public JsonModel testImageUpload() throws Exception {
        JsonModel jsonModel = JsonModel.newInstance();
        try {
            //获取base64文件编码
            String imgStr = GetImageStr();
            //解析编码
            String filepath = GenerateImage(imgStr,"jpg");
            String result = client.uploadFile(filepath);
            if (!StringUtils.isEmpty(result)) {
                result = imageDomain + "/" + result;
                logger.info(">>>>>>>>>>>>>>>>图片上传成功:" + result);
                jsonModel.success(result);
            } else {
                logger.info(">>>>>>>>>>>>>>>>图片上传失败:");
                jsonModel.error("图片上传失败");
            }
        } catch (Exception e) {
            logger.info("图片上传异常:" + e.getMessage());
            jsonModel.error();
        }
        return jsonModel;
    }

    /**
     * 将图片文件转化为字节数组字符串，并对其进行Base64编码处理
     *
     * @return
     */
    public String GetImageStr() {
        String imgFile = "/mnt/newdatadrive/data/custom/autodeploy/client/releasecode/mai/mai@beta@2020010900000000/test.jpg";//待处理的图片
        InputStream in = null;
        byte[] data = null;
        //读取图片字节数组
        try {
            in = new FileInputStream(imgFile);
            data = new byte[in.available()];
            in.read(data);
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        //对字节数组Base64编码
        BASE64Encoder encoder = new BASE64Encoder();
        logger.info("encoderImageStr--------------:" + encoder.encode(data));
        return encoder.encode(data);//返回Base64编码过的字节数组字符串
    }

    /**
     * 对字节数组字符串进行Base64解码并生成本地图片地址
     *
     * @param imgStr
     * @return
     */
    public String GenerateImage(String imgStr,String imgStrType) {
        if (imgStr == null) {//图像数据为空
            return null;
        }
        BASE64Decoder decoder = new BASE64Decoder();
        try {
            //Base64解码
            byte[] b = decoder.decodeBuffer(imgStr);
            for (int i = 0; i < b.length; ++i) {
                if (b[i] < 0) {//调整异常数据
                    b[i] += 256;
                }
            }
            //生成jpeg图片
//            String path = ClassUtils.getDefaultClassLoader().getResource("").getPath();
            String file = new Random().nextInt(99999999) + imgStrType;
            String filePath = "/mnt/newdatadrive/data/runtime/jar-instance/mall/tempImage/" + file;
            logger.info("1--------------filePath：" + filePath);
            //新生成的图片
            OutputStream out = new FileOutputStream(filePath);
            out.write(b);
            out.flush();
            out.close();
            File file2 = new File(filePath);
            InputStream inStream = new FileInputStream(file2);
//			String suffix = FileUtil.getImageFileType(file2);
//			logger.info("3--------------"+suffix);
//			List<ImageSize> sizeList = new ArrayList<>();
//			ImageUploadInfo imageUploadInfo = com.caimei.dfs.image.ImageUpload.resizeUpload(inStream, suffix, sizeList, null);
//			String imageURL = imageDomain + imageUploadInfo.getSource();
//			String imageURL = "";
//			logger.info("上传图片成功！"+imageURL);
            return filePath;
        } catch (Exception e) {
            logger.info("上传图片异常！");
            return null;
        }
    }

    public String getDirName() {
        return dirName;
    }

    public void setDirName(String dirName) {
        this.dirName = dirName;
    }

    public File getFileMaterial() {
        return fileMaterial;
    }

    public void setFileMaterial(File fileMaterial) {
        this.fileMaterial = fileMaterial;
    }

    public Hashtable<String, String> getJsonResult() {
        return jsonResult;
    }

    public void setJsonResult(Hashtable<String, String> jsonResult) {
        this.jsonResult = jsonResult;
    }

}
